@@ -12,13 +12,7 @@
     (read s)))
 (host-cload-stem "src/compiler/generic/genesis" nil)
 
-(defparameter *preload-object-file*
-  (or (stem-remap-target
-       "obj/from-xc/src/assembly/target/tramps.assem-obj")
-      nil))
-
 (sb-cold:genesis :object-file-names *target-object-file-names*
-               :preload-file *preload-object-file*
                :c-header-dir-name "output/genesis-2"
                :symbol-table-file-name "src/runtime/sbcl.nm"
                :core-file-name "output/cold-sbcl.core"
