@@ -78,6 +78,8 @@
      ,@forms
      (dolist (f wrapped-functions) (unencapsulate f '!cold-init))))
 
+(defun !c-runtime-noinform-p () (/= (extern-alien "lisp_startup_options" char) 0))
+
 ;;; called when a cold system starts up
 (defun !cold-init (&aux real-choose-symbol-out-fun)
   "Give the world a shove and hope it spins."
@@ -90,7 +92,8 @@
   (setq *error-output* (!make-cold-stderr-stream)
                       *standard-output* *error-output*
                       *trace-output* *error-output*)
-  (write-string "COLD-INIT... ")
+  (unless (!c-runtime-noinform-p)
+    (write-string "COLD-INIT... "))
 
   ;; Assert that FBOUNDP doesn't choke when its answer is NIL.
   ;; It was fine if T because in that case the legality of the arg is certain.
@@ -178,8 +181,11 @@
   ;; fixups be done separately? Wouldn't that be clearer and better?
   ;; -- WHN 19991204
   (/show0 "doing cold toplevel forms and fixups")
-  (progn (write `("Length(TLFs)= " ,(length *!cold-toplevels*)))
-         (terpri))
+  (unless (!c-runtime-noinform-p)
+    (write `("Length(TLFs)= " ,(length *!cold-toplevels*)))
+    (terpri))
+  ;; only the basic external formats are present at this point.
+  (setq sb!impl::*default-external-format* :latin-1)
 
   (!with-init-wrappers
     (loop for index-in-cold-toplevels from 0
@@ -276,6 +282,8 @@
   (/show0 "enabling internal errors")
   (setf (extern-alien "internal_errors_enabled" int) 1)
 
+  (show-and-call sb!disassem::!compile-inst-printers)
+
   ;; Toggle some readonly bits
   (dovector (sc sb!c:*backend-sc-numbers*)
     (when sc
@@ -294,18 +302,15 @@
   (/show0 "back from first GC")
 
   ;; The show is on.
-  (terpri)
   (/show0 "going into toplevel loop")
   (handling-end-of-the-world
     (toplevel-init)
     (critically-unreachable "after TOPLEVEL-INIT")))
 
-(define-deprecated-function :early "1.0.56.55" quit (exit sb!thread:abort-thread)
-    (&key recklessly-p (unix-status 0))
-  (if (or recklessly-p (sb!thread:main-thread-p))
-      (exit :code unix-status :abort recklessly-p)
-      (sb!thread:abort-thread))
-  (critically-unreachable "after trying to die in QUIT"))
+(defun quit (&key recklessly-p (unix-status 0))
+  "Calls (SB-EXT:EXIT :CODE UNIX-STATUS :ABORT RECKLESSLY-P),
+see documentation for SB-EXT:EXIT."
+  (exit :code unix-status :abort recklessly-p))
 
 (declaim (ftype (sfunction (&key (:code (or null exit-code))
                                  (:timeout (or null real))
