@@ -889,7 +889,8 @@ the current thread are replaced with dummy objects which can safely escape."
 
 (defun invoke-debugger (condition)
   "Enter the debugger."
-  (let ((*stack-top-hint* (resolve-stack-top-hint)))
+  (let ((*stack-top-hint* (resolve-stack-top-hint))
+        (sb!impl::*deadline* nil))
     ;; call *INVOKE-DEBUGGER-HOOK* first, so that *DEBUGGER-HOOK* is not
     ;; called when the debugger is disabled
     (run-hook '*invoke-debugger-hook* condition)
@@ -967,29 +968,41 @@ the current thread are replaced with dummy objects which can safely escape."
       ;; older debugger code which was written to do i/o on whatever
       ;; stream was in fashion at the time, and not all of it has
       ;; been converted to behave this way. -- WHN 2000-11-16)
-
-      (unwind-protect
-           (let (;; We used to bind *STANDARD-OUTPUT* to *DEBUG-IO*
-                 ;; here as well, but that is probably bogus since it
-                 ;; removes the users ability to do output to a redirected
-                 ;; *S-O*. Now we just rebind it so that users can temporarily
-                 ;; frob it. FIXME: This and other "what gets bound when"
-                 ;; behaviour should be documented in the manual.
-                 (*standard-output* *standard-output*)
-                 ;; This seems reasonable: e.g. if the user has redirected
-                 ;; *ERROR-OUTPUT* to some log file, it's probably wrong
-                 ;; to send errors which occur in interactive debugging to
-                 ;; that file, and right to send them to *DEBUG-IO*.
-                 (*error-output* *debug-io*))
-             (unless (typep condition 'step-condition)
-               (when *debug-beginner-help-p*
-                 (format *debug-io*
-                         "~%~@<Type HELP for debugger help, or ~
+      (flet ((debug ()
+               (unwind-protect
+                    (let ( ;; We used to bind *STANDARD-OUTPUT* to *DEBUG-IO*
+                          ;; here as well, but that is probably bogus since it
+                          ;; removes the users ability to do output to a redirected
+                          ;; *S-O*. Now we just rebind it so that users can temporarily
+                          ;; frob it. FIXME: This and other "what gets bound when"
+                          ;; behaviour should be documented in the manual.
+                          (*standard-output* *standard-output*)
+                          ;; This seems reasonable: e.g. if the user has redirected
+                          ;; *ERROR-OUTPUT* to some log file, it's probably wrong
+                          ;; to send errors which occur in interactive debugging to
+                          ;; that file, and right to send them to *DEBUG-IO*.
+                          (*error-output* *debug-io*))
+                      (unless (typep condition 'step-condition)
+                        (when *debug-beginner-help-p*
+                          (format *debug-io*
+                                  "~%~@<Type HELP for debugger help, or ~
                                (SB-EXT:EXIT) to exit from SBCL.~:@>~2%"))
-               (show-restarts *debug-restarts* *debug-io*))
-             (internal-debug))
-        (when background-p
-          (sb!thread::release-foreground))))))
+                        (show-restarts *debug-restarts* *debug-io*))
+                      (internal-debug))
+                 (when background-p
+                   (sb!thread:release-foreground)))))
+        (if (find 'abort *debug-restarts* :key #'restart-name)
+            (debug)
+            (restart-case (let* ((restarts (compute-restarts condition))
+                                 ;; Put the ABORT restart last,
+                                 ;; as if it were provided by a toplevel function.
+                                 (*debug-restarts* (nconc (cdr restarts)
+                                                          (list (car restarts)))))
+                            (debug))
+              (abort ()
+                :report (lambda (stream)
+                          (format stream "~@<Exit from the current thread.~@:>"))
+                (sb!thread:abort-thread :allow-exit t))))))))
 
 ;;; this function is for use in *INVOKE-DEBUGGER-HOOK* when ordinary
 ;;; ANSI behavior has been suppressed by the "--disable-debugger"
