@@ -28,6 +28,11 @@
 (def!type hash ()
   `(integer 0 ,max-hash))
 
+;;; A number that can represent an index into a vector, including
+;;; one-past-the-end
+(deftype array-range ()
+  '(integer 0 #.sb!xc:array-dimension-limit))
+
 ;;; a type used for indexing into sequences, and for related
 ;;; quantities like lengths of lists and other sequences.
 ;;;
@@ -76,7 +81,7 @@
 ;;; The smallest power of two that is equal to or greater than X.
 (declaim (inline power-of-two-ceiling))
 (defun power-of-two-ceiling (x)
-  (declare (index x))
+  (declare (type index x))
   (ash 1 (integer-length (1- x))))
 
 (def!type load/store-index (scale lowtag min-offset
@@ -1034,27 +1039,6 @@ NOTE: This interface is experimental and subject to change."
   (def-constantly-fun constantly-nil nil)
   (def-constantly-fun constantly-0 0))
 
-;;; If X is a symbol, see whether it is present in *FEATURES*. Also
-;;; handle arbitrary combinations of atoms using NOT, AND, OR.
-(defun featurep (x)
-  (typecase x
-    (cons
-     (case (car x)
-       ((:not not)
-        (cond
-          ((cddr x)
-           (error "too many subexpressions in feature expression: ~S" x))
-          ((null (cdr x))
-           (error "too few subexpressions in feature expression: ~S" x))
-          (t (not (featurep (cadr x))))))
-       ((:and and) (every #'featurep (cdr x)))
-       ((:or or) (some #'featurep (cdr x)))
-       (t
-        (error "unknown operator in feature expression: ~S." x))))
-    (symbol (not (null (memq x *features*))))
-    (t
-      (error "invalid feature expression: ~S" x))))
-
 
 ;;;; utilities for two-VALUES predicates
 
@@ -1447,7 +1431,7 @@ NOTE: This interface is experimental and subject to change."
   #+sb-xc-host (declare (ignore software version name replacement-spec))
   #-sb-xc-host
   (setf (fdefinition name)
-        (sb!impl::set-closure-name
+        (set-closure-name
          (lambda (&rest args)
            (declare (ignore args))
            (deprecation-error software version 'function name replacement-spec))
@@ -1718,6 +1702,19 @@ to :INTERPRET, an interpreter will be used.")
              (sorted (stable-sort wrapped comparator :key #'cdr)))
         (map-into sorted #'car sorted))))
 
+(declaim (inline schwartzian-stable-sort-vector))
+(defun schwartzian-stable-sort-vector (vector comparator &key key)
+  (if (null key)
+      (stable-sort (copy-seq vector) comparator)
+      (let* ((key (if (functionp key)
+                      key
+                      (symbol-function key)))
+             (wrapped (map 'vector (lambda (x)
+                                     (cons x (funcall key x)))
+                           vector))
+             (sorted (stable-sort wrapped comparator :key #'cdr)))
+        (map-into sorted #'car sorted))))
+
 ;;; Just like WITH-OUTPUT-TO-STRING but doesn't close the stream,
 ;;; producing more compact code.
 (defmacro with-simple-output-to-string
