@@ -3,27 +3,24 @@
 (in-package "SB!VM")
 
 
-(define-alien-type os-context-t (struct os-context-t-struct))
-(define-alien-type os-context-register-t unsigned-long-long)
-
-
-;;;; MACHINE-TYPE
-
+#-sb-xc-host
 (defun machine-type ()
   "Returns a string describing the type of the local machine."
   "MIPS")
 
 ;;;; FIXUP-CODE-OBJECT
 
-(defun fixup-code-object (code offset value kind)
+(defconstant-eqx +fixup-kinds+ #(:absolute :jmp :lui :addi) #'equalp)
+(!with-bigvec-or-sap
+(defun fixup-code-object (code offset value kind flavor)
   (declare (type index offset))
+  (declare (ignore flavor))
   (unless (zerop (rem offset n-word-bytes))
     (error "Unaligned instruction?  offset=#x~X." offset))
-  (without-gcing
-   (let ((sap (%primitive sb!c::code-instructions code)))
-     (ecase kind
+  (let ((sap (code-instructions code)))
+    (ecase kind
        (:absolute
-        (setf (sap-ref-32 sap offset) fixup))
+        (setf (sap-ref-32 sap offset) value))
        (:jump
         (aver (zerop (ash value -28)))
         (setf (ldb (byte 26 0) (sap-ref-32 sap offset))
@@ -33,49 +30,22 @@
               (ash (1+ (ash value -15)) -1)))
        (:addi
         (setf (ldb (byte 16 0) (sap-ref-32 sap offset))
-              (ldb (byte 16 0) value)))))))
+              (ldb (byte 16 0) value)))))
+  nil))
 
 
-(define-alien-routine ("os_context_pc_addr" context-pc-addr)
-    (* os-context-register-t)
-  (context (* os-context-t) :in))
-
-(defun context-pc (context)
-  (declare (type (alien (* os-context-t)) context))
-  (int-sap (deref (context-pc-addr context))))
-
-(define-alien-routine ("os_context_register_addr" context-register-addr)
-    (* os-context-register-t)
-  (context (* os-context-t) :in)
-  (index int :in))
+#-sb-xc-host (progn
 
 (define-alien-routine ("os_context_bd_cause" context-bd-cause-int)
     unsigned-int
   (context (* os-context-t) :in))
 
-;;; FIXME: Should this and CONTEXT-PC be INLINE to reduce consing?
-;;; (Are they used in anything time-critical, or just the debugger?)
-(defun context-register (context index)
-  (declare (type (alien (* os-context-t)) context))
-  (let ((addr (context-register-addr context index)))
-    (declare (type (alien (* os-context-register-t)) addr))
-    ;; The 32-bit Linux ABI uses 64-bit slots for register storage in
-    ;; the context structure.  At least some hardware sign extends
-    ;; 32-bit values in these registers, leading to badness in the
-    ;; debugger.  Mask it down here.
-    (mask-field (byte 32 0) (deref addr))))
-
-(defun %set-context-register (context index new)
-  (declare (type (alien (* os-context-t)) context))
-  (let ((addr (context-register-addr context index)))
-    (declare (type (alien (* os-context-register-t)) addr))
-    (setf (deref addr) new)))
-
 ;;; This is like CONTEXT-REGISTER, but returns the value of a float
 ;;; register. FORMAT is the type of float to return.
 
 ;;; FIXME: Whether COERCE actually knows how to make a float out of a
 ;;; long is another question. This stuff still needs testing.
+(define-alien-type os-context-register-t unsigned-long-long)
 (define-alien-routine ("os_context_fpregister_addr" context-float-register-addr)
     (* os-context-register-t)
   (context (* os-context-t) :in)
@@ -120,3 +90,4 @@
     (values error-number
             (sb!kernel::decode-internal-error-args (sap+ pc (1+ offset))
                                                    error-number))))
+) ; end PROGN
