@@ -164,7 +164,9 @@
                 ((and (consp next) (eq :home (car next)))
                  (pieces "~")
                  (pieces (second next)))
-                ((and (plusp (length next)) (char= #\~ (char next 0)))
+                ((and (stringp next)
+                      (plusp (length next))
+                      (char= #\~ (char next 0)))
                  ;; The only place we need to escape the tilde.
                  (pieces "\\")
                  (pieces next))
