@@ -11,61 +11,6 @@
 
 (in-package "SB!UNIX")
 
-(defmacro with-interrupt-bindings (&body body)
-  `(let*
-       ;; KLUDGE: Whatever is on the PCL stacks before the interrupt
-       ;; handler runs doesn't really matter, since we're not on the
-       ;; same call stack, really -- and if we don't bind these (esp.
-       ;; the cache one) we can get a bogus metacircle if an interrupt
-       ;; handler calls a GF that was being computed when the interrupt
-       ;; hit.
-       ((sb!pcl::*cache-miss-values-stack* nil)
-        (sb!pcl::*dfun-miss-gfs-on-stack* nil))
-     ,@body))
-
-;;; Evaluate CLEANUP-FORMS iff PROTECTED-FORM does a non-local exit.
-(defmacro nlx-protect (protected-form &rest cleanup-froms)
-  (with-unique-names (completep)
-    `(let ((,completep nil))
-       (without-interrupts
-         (unwind-protect
-              (progn
-                (allow-with-interrupts
-                  ,protected-form)
-                (setq ,completep t))
-           (unless ,completep
-             ,@cleanup-froms))))))
-
-(defun invoke-interruption (function)
-  (without-interrupts
-    ;; Reset signal mask: the C-side handler has blocked all
-    ;; deferrable signals before funcalling into lisp. They are to be
-    ;; unblocked the first time interrupts are enabled. With this
-    ;; mechanism there are no extra frames on the stack from a
-    ;; previous signal handler when the next signal is delivered
-    ;; provided there is no WITH-INTERRUPTS.
-    (let ((*unblock-deferrables-on-enabling-interrupts-p* t)
-          (sb!debug:*stack-top-hint* (or sb!debug:*stack-top-hint* 'invoke-interruption)))
-      (with-interrupt-bindings
-        (sb!thread::without-thread-waiting-for (:already-without-interrupts t)
-          (allow-with-interrupts
-            (nlx-protect (funcall function)
-                         ;; We've been running with deferrables
-                         ;; blocked in Lisp called by a C signal
-                         ;; handler. If we return normally the sigmask
-                         ;; in the interrupted context is restored.
-                         ;; However, if we do an nlx the operating
-                         ;; system will not restore it for us.
-                         (when *unblock-deferrables-on-enabling-interrupts-p*
-                           ;; This means that storms of interrupts
-                           ;; doing an nlx can still run out of stack.
-                           (unblock-deferrable-signals)))))))))
-
-(defmacro in-interruption ((&key) &body body)
-  "Convenience macro on top of INVOKE-INTERRUPTION."
-  `(dx-flet ((interruption () ,@body))
-     (invoke-interruption #'interruption)))
-
 ;;;; system calls that deal with signals
 
 ;;; Send the signal SIGNAL to the process with process id PID. SIGNAL
@@ -92,21 +37,12 @@
 ;;; doing things the SBCL way and moving this kind of C-level work
 ;;; down to C wrapper functions.)
 
-(declaim (inline %unblock-deferrable-signals %unblock-gc-signals))
-(define-alien-routine ("unblock_deferrable_signals"
-                       %unblock-deferrable-signals)
-  void
-  (where unsigned-long)
-  (old unsigned-long))
 #!-sb-safepoint
 (define-alien-routine ("unblock_gc_signals" %unblock-gc-signals)
     void
   (where unsigned-long)
   (old unsigned-long))
 
-(defun unblock-deferrable-signals ()
-  (%unblock-deferrable-signals 0 0))
-
 #!-sb-safepoint
 (defun unblock-gc-signals ()
   (%unblock-gc-signals 0 0))
@@ -171,18 +107,20 @@
 ;;;; Rather, the signal spawns off a fresh native thread, which calls
 ;;;; into lisp with a fake context through this callback:
 
-#!+(and sb-safepoint-strictly (not win32))
+#!+sb-safepoint-strictly
 (defun signal-handler-callback (run-handler signal args)
   ;; SAPs are dx allocated, close over the values, not the SAPs.
-  (let ((info (sap-ref-sap args 0))
+  (let ((thread (without-gcing
+                  ;; Hold off GCing until *current-thread* is set up
+                  (setf sb!thread:*current-thread*
+                        (sb!thread::make-signal-handling-thread :name "signal handler"
+                                                                :signal-number signal))))
+        (info (sap-ref-sap args 0))
         (context (sap-ref-sap args sb!vm:n-word-bytes)))
-    (sb!thread::initial-thread-function-trampoline
-     (sb!thread::make-signal-handling-thread :name "signal handler"
-                                             :signal-number signal)
-     nil (lambda ()
-           (funcall run-handler signal info context))
-     nil
-     nil nil nil nil)))
+    (dx-flet ((callback ()
+                (funcall run-handler signal info context)))
+      (sb!thread::initial-thread-function-trampoline thread nil
+                                                     #'callback nil))))
 
 
 ;;;; default LISP signal handlers
@@ -302,8 +240,3 @@
 ;;; extract si_code from siginfo_t
 (define-alien-routine ("siginfo_code" siginfo-code) int
   (info system-area-pointer))
-
-;;; CMU CL comment:
-;;;   Magically converted by the compiler into a break instruction.
-(defun receive-pending-interrupt ()
-  (receive-pending-interrupt))
