@@ -10,26 +10,6 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
-
-;;;; interfaces to IR2 conversion
-
-;;; Return a wired TN describing the N'th full call argument passing
-;;; location.
-(defun standard-arg-location (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-wired-tn *backend-t-primitive-type*
-                     register-arg-scn
-                     (elt *register-arg-offsets* n))
-      (make-wired-tn *backend-t-primitive-type*
-                     control-stack-arg-scn n)))
-
-(defun standard-arg-location-sc (n)
-  (declare (type unsigned-byte n))
-  (if (< n register-arg-count)
-      (make-sc-offset register-arg-scn
-                      (nth n *register-arg-offsets*))
-      (make-sc-offset control-stack-arg-scn n)))
 
 (defconstant arg-count-sc (make-sc-offset immediate-arg-scn nargs-offset))
 (defconstant closure-sc (make-sc-offset descriptor-reg-sc-number lexenv-offset))
@@ -40,8 +20,8 @@
 ;;; restricted by a desire to use a subroutine call instruction.
 (defun make-return-pc-passing-location (standard)
   (if standard
-      (make-wired-tn *backend-t-primitive-type* register-arg-scn lra-offset)
-      (make-restricted-tn *backend-t-primitive-type* register-arg-scn)))
+      (make-wired-tn *backend-t-primitive-type* descriptor-reg-sc-number lra-offset)
+      (make-restricted-tn *backend-t-primitive-type* descriptor-reg-sc-number)))
 
 ;;; This is similar to MAKE-RETURN-PC-PASSING-LOCATION, but makes a
 ;;; location to pass OLD-FP in. This is (obviously) wired in the
@@ -73,46 +53,14 @@
 ;;; passed when we are using non-standard conventions.
 (defun make-arg-count-location ()
   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nargs-offset))
-
-
-;;; Make a TN to hold the number-stack frame pointer. This is
-;;; allocated once per component, and is component-live.
-(defun make-nfp-tn ()
-  (component-live-tn
-   (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nfp-offset)))
-
-(defun make-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-(defun make-number-stack-pointer-tn ()
-  (make-normal-tn *fixnum-primitive-type*))
-
-;;; Return a list of TNs that can be used to represent an
-;;; unknown-values continuation within a function.
-(defun make-unknown-values-locations ()
-  (list (make-stack-pointer-tn)
-        (make-normal-tn *fixnum-primitive-type*)))
-
-
-;;; This function is called by the ENTRY-ANALYZE phase, allowing
-;;; VM-dependent initialization of the IR2-COMPONENT structure. We
-;;; push placeholder entries in the CONSTANTS to leave room for
-;;; additional noise in the code object header.
-(defun select-component-format (component)
-  (declare (type component component))
-  (dotimes (i code-constants-offset)
-    (vector-push-extend nil
-                        (ir2-component-constants (component-info component))))
-  (values))
-
 
 ;;;; frame hackery
 
 ;;; Return the number of bytes needed for the current non-descriptor
-;;; stack frame. Non-descriptor stack frames must be multiples of 8
-;;; bytes on the PMAX.
+;;; stack frame. Non-descriptor stack frames must be multiples of 16
+;;; bytes to preserve alien stack alignment.
 (defun bytes-needed-for-non-descriptor-stack-frame ()
-  (* (logandc2 (1+ (sb-allocated-size 'non-descriptor-stack)) 1)
+  (* (logandc2 (+ 3 (sb-allocated-size 'non-descriptor-stack)) 3)
      n-word-bytes))
 
 ;;; This is used for setting up the Old-FP in local call.
@@ -130,7 +78,7 @@
   (:generator 1
     (let ((nfp (current-nfp-tn vop)))
       (when nfp
-        (inst addq nfp (bytes-needed-for-non-descriptor-stack-frame) val)))))
+        (inst lda val (bytes-needed-for-non-descriptor-stack-frame) nfp)))))
 
 ;;; Accessing a slot from an earlier stack frame is definite hackery.
 (define-vop (ancestor-frame-ref)
@@ -180,8 +128,7 @@
           cfp-tn)
     (let ((nfp (current-nfp-tn vop)))
       (when nfp
-        (inst subq nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)
+        (inst lda nsp-tn (- (bytes-needed-for-non-descriptor-stack-frame)) nsp-tn)
         (move nsp-tn nfp)))))
 
 (define-vop (allocate-frame)
@@ -195,8 +142,7 @@
           (* n-word-bytes (sb-allocated-size 'control-stack))
           csp-tn)
     (when (ir2-physenv-number-stack-p callee)
-      (inst subq nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
-            nsp-tn)
+      (inst lda nsp-tn (- (bytes-needed-for-non-descriptor-stack-frame)) nsp-tn)
       (move nsp-tn nfp))))
 
 ;;; Allocate a partial frame for passing stack arguments in a full
@@ -573,8 +519,8 @@ default-value-8
     (move cfp-tn csp-tn)
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
     (inst subq return-pc-temp (- other-pointer-lowtag n-word-bytes) lip)
     (move ocfp-temp cfp-tn)
     (inst ret zero-tn lip 1)))
@@ -627,10 +573,12 @@ default-value-8
      (:args
       ,@(unless (eq return :tail)
           '((new-fp :scs (any-reg) :to :eval)))
-
-      ,(if named
-           '(name :target name-pass)
-           '(arg-fun :target lexenv))
+      ,@(case named
+          ((nil)
+           '((arg-fun :target lexenv)))
+          (:direct)
+          (t
+           '((name :target name-pass))))
 
       ,@(when (eq return :tail)
           '((ocfp :target ocfp-pass)
@@ -649,6 +597,7 @@ default-value-8
      (:vop-var vop)
      (:info ,@(unless (or variable (eq return :tail)) '(arg-locs))
             ,@(unless variable '(nargs))
+            ,@(when (eq named :direct) '(fun))
             ,@(when (eq return :fixed) '(nvals))
             step-instrumenting)
 
@@ -672,19 +621,20 @@ default-value-8
                   :to :eval)
                  return-pc-pass)
 
-     ,@(if named
-         `((:temporary (:sc descriptor-reg :offset fdefn-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       name-pass))
-
-         `((:temporary (:sc descriptor-reg :offset lexenv-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       lexenv)
-           #!-gengc
-           (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
-                       function)))
+     ,@(case named
+         ((t)
+          `((:temporary (:sc descriptor-reg :offset fdefn-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        name-pass)))
+         ((nil)
+          `((:temporary (:sc descriptor-reg :offset lexenv-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        lexenv)
+            #!-gengc
+            (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
+                        function))))
 
      (:temporary (:sc any-reg :offset nargs-offset :to :eval)
                  nargs-pass)
@@ -765,9 +715,9 @@ default-value-8
                                              word-shift)
                                          cfp-tn))))
                               (:frob-nfp
-                               (inst addq cur-nfp
+                               (inst lda nsp-tn
                                      (bytes-needed-for-non-descriptor-stack-frame)
-                                     nsp-tn)))
+                                     cur-nfp)))
                             `(#!-gengc
                               (:comp-lra
                                (inst compute-lra-from-code
@@ -783,50 +733,53 @@ default-value-8
                                          (move new-fp cfp-tn)
                                          (move csp-tn cfp-tn))))))
                       ((nil))))))
-
-           ,@(if named
-                 `((sc-case name
-                     (descriptor-reg (move name name-pass))
-                     (control-stack
-                      (inst ldl name-pass
-                            (ash (tn-offset name) word-shift) cfp-tn)
-                      (do-next-filler))
-                     (constant
-                      (inst ldl name-pass
-                            (- (ash (tn-offset name) word-shift)
-                               other-pointer-lowtag) code-tn)
-                      (do-next-filler)))
-                   (inst ldl entry-point
-                         (- (ash fdefn-raw-addr-slot word-shift)
-                            other-pointer-lowtag) name-pass)
-                   (do-next-filler))
-                 `((sc-case arg-fun
-                     (descriptor-reg (move arg-fun lexenv))
-                     (control-stack
-                      (inst ldl lexenv
-                            (ash (tn-offset arg-fun) word-shift) cfp-tn)
-                      (do-next-filler))
-                     (constant
-                      (inst ldl lexenv
-                            (- (ash (tn-offset arg-fun) word-shift)
-                               other-pointer-lowtag) code-tn)
-                      (do-next-filler)))
-                   #!-gengc
-                   (inst ldl function
-                         (- (ash closure-fun-slot word-shift)
-                            fun-pointer-lowtag) lexenv)
-                   #!-gengc
-                   (do-next-filler)
-                   #!-gengc
-                   (inst addq function
-                         (- (ash simple-fun-code-offset word-shift)
-                            fun-pointer-lowtag) entry-point)
-                   #!+gengc
-                   (inst ldl entry-point
-                         (- (ash closure-entry-point-slot word-shift)
-                            fun-pointer-lowtag) lexenv)
-                   #!+gengc
-                   (do-next-filler)))
+           ,@(case named
+               ((t)
+                `((sc-case name
+                    (descriptor-reg (move name name-pass))
+                    (control-stack
+                     (inst ldl name-pass
+                           (ash (tn-offset name) word-shift) cfp-tn)
+                     (do-next-filler))
+                    (constant
+                     (inst ldl name-pass
+                           (- (ash (tn-offset name) word-shift)
+                              other-pointer-lowtag) code-tn)
+                     (do-next-filler)))
+                  (inst ldl entry-point
+                        (- (ash fdefn-raw-addr-slot word-shift)
+                           other-pointer-lowtag) name-pass)
+                  (do-next-filler)))
+               ((nil)
+                `((sc-case arg-fun
+                    (descriptor-reg (move arg-fun lexenv))
+                    (control-stack
+                     (inst ldl lexenv
+                           (ash (tn-offset arg-fun) word-shift) cfp-tn)
+                     (do-next-filler))
+                    (constant
+                     (inst ldl lexenv
+                           (- (ash (tn-offset arg-fun) word-shift)
+                              other-pointer-lowtag) code-tn)
+                     (do-next-filler)))
+                  #!-gengc
+                  (inst ldl function
+                        (- (ash closure-fun-slot word-shift)
+                           fun-pointer-lowtag) lexenv)
+                  #!-gengc
+                  (do-next-filler)
+                  #!-gengc
+                  (inst addq function
+                        (- (ash simple-fun-code-offset word-shift)
+                           fun-pointer-lowtag) entry-point)
+                  #!+gengc
+                  (inst ldl entry-point
+                        (- (ash closure-entry-point-slot word-shift)
+                           fun-pointer-lowtag) lexenv)
+                  #!+gengc
+                  (do-next-filler)))
+               (:direct
+                `((inst ldl entry-point (static-fun-offset fun) null-tn))))
            (loop
              (if (cdr filler)
                  (do-next-filler)
@@ -852,10 +805,13 @@ default-value-8
 
 (define-full-call call nil :fixed nil)
 (define-full-call call-named t :fixed nil)
+(define-full-call static-call-named :direct :fixed nil)
 (define-full-call multiple-call nil :unknown nil)
 (define-full-call multiple-call-named t :unknown nil)
+(define-full-call static-multiple-call-named :direct :unknown nil)
 (define-full-call tail-call nil :tail nil)
 (define-full-call tail-call-named t :tail nil)
+(define-full-call static-tail-call-named :direct :tail nil)
 
 (define-full-call call-variable nil :fixed t)
 (define-full-call multiple-call-variable nil :unknown t)
@@ -889,8 +845,8 @@ default-value-8
     ;; Clear the number stack if anything is there.
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
 
     ;; And jump to the assembly-routine that does the bliting.
     (inst li (make-fixup 'tail-call-variable :assembly-routine) temp)
@@ -913,8 +869,8 @@ default-value-8
     ;; Clear the number stack.
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
     ;; Clear the control stack, and restore the frame pointer.
     (move cfp-tn csp-tn)
     (move ocfp cfp-tn)
@@ -962,8 +918,8 @@ default-value-8
     ;; Clear the number stack.
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
-        (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-              nsp-tn)))
+        (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+              cur-nfp)))
     ;; Establish the values pointer and values count.
     (move cfp-tn val-ptr)
     (inst li (fixnumize nvals) nargs)
@@ -1014,8 +970,8 @@ default-value-8
       ;; Clear the number stack.
       (let ((cur-nfp (current-nfp-tn vop)))
         (when cur-nfp
-          (inst addq cur-nfp (bytes-needed-for-non-descriptor-stack-frame)
-                nsp-tn)))
+          (inst lda nsp-tn (bytes-needed-for-non-descriptor-stack-frame)
+                cur-nfp)))
 
       ;; Check for the single case.
       (inst li (fixnumize 1) a0)
@@ -1198,43 +1154,22 @@ default-value-8
     (inst subq supplied (fixnumize fixed) count)
     (inst subq csp-tn count context)))
 
-;;; Signal wrong argument count error if NARGS isn't equal to COUNT.
-#!-precise-arg-count-error
-(define-vop (verify-arg-count)
-  (:policy :fast-safe)
-  (:translate sb!c::%verify-arg-count)
-  (:args (nargs :scs (any-reg)))
-  (:arg-types positive-fixnum (:constant t))
-  (:temporary (:scs (any-reg) :type fixnum) temp)
-  (:info count)
-  (:vop-var vop)
-  (:save-p :compute-only)
-  (:generator 3
-    (let ((err-lab
-           (generate-error-code vop invalid-arg-count-error nargs)))
-      (cond ((zerop count)
-             (inst bne nargs err-lab))
-            (t
-             (inst subq nargs (fixnumize count) temp)
-             (inst bne temp err-lab))))))
-
-#!+precise-arg-count-error
 (define-vop (verify-arg-count)
   (:policy :fast-safe)
   (:args (nargs :scs (any-reg)))
   (:arg-types positive-fixnum (:constant t) (:constant t))
   (:info min max)
-  (:temporary (:scs (any-reg) :type fixnum) temp)
+  (:temporary (:scs (unsigned-reg) :type fixnum) temp)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
     (let ((err-lab
-            (generate-error-code vop invalid-arg-count-error nargs)))
+            (generate-error-code vop 'invalid-arg-count-error nargs)))
       (cond ((not min)
              (cond ((zerop max)
                     (inst bne nargs err-lab))
                    (t
-                    (inst subq nargs (fixnumize count) temp)
+                    (inst subq nargs (fixnumize max) temp)
                     (inst bne temp err-lab))))
             (max
              (when (plusp min)
@@ -1249,33 +1184,7 @@ default-value-8
              (inst cmpult nargs temp temp)
              (inst bne temp err-lab))))))
 
-;;; various other error signalers
-(macrolet ((frob (name error translate &rest args)
-             `(define-vop (,name)
-                ,@(when translate
-                    `((:policy :fast-safe)
-                      (:translate ,translate)))
-                (:args ,@(mapcar (lambda (arg)
-                                   `(,arg :scs (any-reg descriptor-reg)))
-                                 args))
-                (:vop-var vop)
-                (:save-p :compute-only)
-                (:generator 1000
-                  (error-call vop ,error ,@args)))))
-  (frob arg-count-error invalid-arg-count-error
-    sb!c::%arg-count-error nargs fname)
-  (frob type-check-error object-not-type-error sb!c::%type-check-error
-    object type)
-  (frob layout-invalid-error layout-invalid-error sb!c::%layout-invalid-error
-    object layout)
-  (frob odd-key-args-error odd-key-args-error
-    sb!c::%odd-key-args-error)
-  (frob unknown-key-arg-error unknown-key-arg-error
-    sb!c::%unknown-key-arg-error key)
-  (frob nil-fun-returned-error nil-fun-returned-error nil fun))
-
 ;;; Single-stepping
-
 (define-vop (step-instrument-before-vop)
   (:policy :fast-safe)
   (:vop-var vop)
