@@ -19,7 +19,7 @@
 (macrolet ((defreg (name offset)
              (let ((offset-sym (symbolicate name "-OFFSET")))
                `(eval-when (:compile-toplevel :load-toplevel :execute)
-                  (def!constant ,offset-sym ,offset)
+                  (defconstant ,offset-sym ,offset)
                   (setf (svref *register-names* ,offset-sym)
                         ,(symbol-name name)))))
            (defregset (name &rest regs)
@@ -84,38 +84,14 @@
     a0 a1 a2 a3 a4 a5)
   (defparameter register-arg-names '(a0 a1 a2 a3 a4 a5)))
 
+(!define-storage-bases
 (define-storage-base registers :finite :size 32)
 (define-storage-base float-registers :finite :size 64)
 (define-storage-base control-stack :unbounded :size 8)
 (define-storage-base non-descriptor-stack :unbounded :size 0)
 (define-storage-base constant :non-packed)
 (define-storage-base immediate-constant :non-packed)
-
-;;; a handy macro so we don't have to keep changing all the numbers
-;;; whenever we insert a new storage class.
-
-(defmacro !define-storage-classes (&rest classes)
-  (do ((forms (list 'progn)
-              (let* ((class (car classes))
-                     (sc-name (car class))
-                     (constant-name (intern (concatenate 'simple-string
-                                                         (string sc-name)
-                                                         "-SC-NUMBER"))))
-                (list* `(define-storage-class ,sc-name ,index
-                          ,@(cdr class))
-                       `(def!constant ,constant-name ,index)
-                       ;; (The CMU CL version of this macro did
-                       ;;   `(EXPORT ',CONSTANT-NAME)
-                       ;; here, but in SBCL we try to have package
-                       ;; structure described statically in one
-                       ;; master source file, instead of building it
-                       ;; dynamically by letting all the system code
-                       ;; modify it as the system boots.)
-                       forms)))
-       (index 0 (1+ index))
-       (classes classes (cdr classes)))
-      ((null classes)
-       (nreverse forms))))
+)
 
 (!define-storage-classes
 
@@ -240,8 +216,8 @@
    :save-p t
    :alternate-scs (complex-double-stack))
 
-  ;; A catch or unwind block.
-  (catch-block control-stack :element-size catch-block-size))
+  (catch-block control-stack :element-size catch-block-size)
+  (unwind-block control-stack :element-size unwind-block-size))
 
 ;;; Make some random tns for important registers.
 (macrolet ((defregtn (name sc)
@@ -284,29 +260,29 @@
 (defun immediate-constant-sc (value)
   (typecase value
     ((integer 0 0)
-     (sc-number-or-lose 'zero))
+     zero-sc-number)
     (null
-     (sc-number-or-lose 'null ))
+     null-sc-number)
     ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
          character)
-     (sc-number-or-lose 'immediate ))
+     immediate-sc-number)
     (symbol
      (if (static-symbol-p value)
-         (sc-number-or-lose 'immediate )
+         immediate-sc-number
          nil))
     (single-float
      (if (eql value 0f0)
-         (sc-number-or-lose 'fp-single-zero )
+         fp-single-zero-sc-number
          nil))
     (double-float
      (if (eql value 0d0)
-         (sc-number-or-lose 'fp-double-zero )
+         fp-double-zero-sc-number
          nil))))
 
 (defun boxed-immediate-sc-p (sc)
-  (or (eql sc (sc-number-or-lose 'zero))
-      (eql sc (sc-number-or-lose 'null))
-      (eql sc (sc-number-or-lose 'immediate))))
+  (or (eql sc zero-sc-number)
+      (eql sc null-sc-number)
+      (eql sc immediate-sc-number)))
 
 ;;; A predicate to see if a character can be used as an inline
 ;;; constant (the immediate field in the instruction used is eight
@@ -319,18 +295,18 @@
 ;;;; function call parameters
 
 ;;; the SC numbers for register and stack arguments/return values
-(def!constant immediate-arg-scn (sc-number-or-lose 'any-reg))
-(def!constant control-stack-arg-scn (sc-number-or-lose 'control-stack))
+(defconstant immediate-arg-scn any-reg-sc-number)
+(defconstant control-stack-arg-scn control-stack-sc-number)
 
 (eval-when  (:compile-toplevel :load-toplevel :execute)
 
 ;;; offsets of special stack frame locations
-(def!constant ocfp-save-offset 0)
-(def!constant lra-save-offset 1)
-(def!constant nfp-save-offset 2)
+(defconstant ocfp-save-offset 0)
+(defconstant lra-save-offset 1)
+(defconstant nfp-save-offset 2)
 
 ;;; the number of arguments/return values passed in registers
-(def!constant register-arg-count 6)
+(defconstant register-arg-count 6)
 
 ;;; (Names to use for the argument registers would go here, but there
 ;;; are none.)
@@ -346,7 +322,7 @@
           *register-arg-offsets*))
 
 ;;; This is used by the debugger.
-(def!constant single-value-return-byte-offset 4)
+(defconstant single-value-return-byte-offset 4)
 
 ;;; This function is called by debug output routines that want a
 ;;; pretty name for a TN's location. It returns a thing that can be
