@@ -378,6 +378,8 @@
                   (:variant :double  ,is-=)))))
   (frob < :mi </single-float </double-float nil)
   (frob > :gt >/single-float >/double-float nil)
+  (frob <= :le <=/single-float <=/double-float nil)
+  (frob >= :ge >=/single-float >=/double-float nil)
   (frob = :eq eql/single-float eql/double-float t))
 
 (define-vop (float-compare-zero)
@@ -416,6 +418,8 @@
                   (:variant ,is-=)))))
   (frob < :mi </single-float-zero </double-float-zero nil)
   (frob > :gt >/single-float-zero >/double-float-zero nil)
+  (frob <= :le <=/single-float-zero <=/double-float-zero nil)
+  (frob >= :ge >=/single-float-zero >=/double-float-zero nil)
   (frob = :eq eql/single-float-zero eql/double-float-zero t))
 
 ;;;; Conversion:
