@@ -582,7 +582,8 @@
       (let* ((type (constant-form-value type env))
              (length (1- (length x)))
              (ctype (careful-values-specifier-type type)))
-        (if (csubtypep ctype (specifier-type '(array * (*))))
+        (if (and ctype
+                 (csubtypep ctype (specifier-type '(array * (*)))))
             (multiple-value-bind (type element-type upgraded had-dimensions)
                 (simplify-vector-type ctype)
               (declare (ignore type upgraded))
@@ -1526,7 +1527,7 @@
     (with-unique-names (n-vector)
       `(let ((,n-vector ,vector))
          (truly-the ,elt-type (data-vector-set
-                               (the simple-vector ,n-vector)
+                               (the* (simple-vector :modifying (setf svref)) ,n-vector)
                                (check-bound ,n-vector (length ,n-vector) ,index)
                                (the ,elt-type ,value)))))))
 
