@@ -529,8 +529,9 @@
         (setf (block-type-check block) nil)))
     (dolist (cast (casts))
       (unless (or (bound-cast-p cast)
-                  ;; The block could become deleted by IR1-OPTIMIZE-CAST
+                  ;; The node could become deleted by IR1-OPTIMIZE-CAST
                   ;; called by CAST-TYPE-CHECK
+                  (node-deleted cast)
                   (node-to-be-deleted-p cast))
         (multiple-value-bind (check types) (cast-check-types cast)
           (ecase check
