@@ -15,6 +15,9 @@
 (deftype byte-buffer () '(vector (unsigned-byte 8)))
 (defvar *byte-buffer*)
 (declaim (type byte-buffer *byte-buffer*))
+(defvar *contexts*)
+(declaim (type (vector t) *contexts*))
+
 
 ;;;; debug blocks
 
@@ -26,22 +29,30 @@
 ;;; The LOCATION-INFO structure holds the information what we need
 ;;; about locations which code generation decided were "interesting".
 (defstruct (location-info
-            (:constructor make-location-info (kind label vop))
+            (:constructor make-location-info (kind label vop context))
             (:copier nil))
   ;; The kind of location noted.
   (kind nil :type location-kind)
+  (context nil)
   ;; The label pointing to the interesting code location.
   (label nil :type (or label index null))
   ;; The VOP that emitted this location (for node, save-set, ir2-block, etc.)
   (vop nil :type vop))
 
+(def!struct (restart-location
+            (:constructor make-restart-location (&optional label tn))
+            (:copier nil))
+  (label nil :type (or null label))
+  (tn nil :type (or null tn) :read-only t))
+(!set-load-form-method restart-location (:xc :target) :ignore-it)
+
 ;;; This is called during code generation in places where there is an
 ;;; "interesting" location: someplace where we are likely to end up
 ;;; in the debugger, and thus want debug info.
-(defun note-debug-location (vop label kind)
+(defun note-debug-location (vop label kind &optional context)
   (declare (type vop vop) (type (or label null) label)
            (type location-kind kind))
-  (let ((location (make-location-info kind label vop)))
+  (let ((location (make-location-info kind label vop context)))
     (setf (ir2-block-locations (vop-block vop))
           (nconc (ir2-block-locations (vop-block vop))
                  (list location)))
@@ -79,10 +90,10 @@
 ;;; variables dumped, compute a bit-vector representing the set of
 ;;; live variables. If the TN is environment-live, we only mark it as
 ;;; live when it is in scope at NODE.
+(declaim (ftype (sfunction (local-tn-bit-vector node ir2-block hash-table (or null vop))
+                           simple-bit-vector)
+                compute-live-vars))
 (defun compute-live-vars (live node block var-locs vop)
-  (declare (type ir2-block block) (type local-tn-bit-vector live)
-           (type hash-table var-locs) (type node node)
-           (type (or vop null) vop))
   (let ((res (make-array (logandc2 (+ (hash-table-count var-locs) 7) 7)
                          :element-type 'bit
                          :initial-element 0))
@@ -106,142 +117,150 @@
 (defvar *previous-location*)
 (declaim (type index *previous-location*))
 
+(defun encode-restart-location (location x)
+  (typecase x
+    (restart-location
+     (let ((offset (- (label-position (restart-location-label x))
+                      location))
+           (tn (restart-location-tn x))
+           (registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
+       (if tn
+           (the fixnum (logior (ash offset registers-size)
+                               (tn-offset tn)))
+           offset)))
+    (t
+     x)))
+
+(defun decode-restart-location (x)
+  (declare (fixnum x))
+  (let ((registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
+    (values (make-sc-offset sb!vm:descriptor-reg-sc-number
+                            (ldb (byte registers-size 0) x))
+            (ash x (- registers-size)))))
+
 ;;; Dump a compiled debug-location into *BYTE-BUFFER* that describes
 ;;; the code/source map and live info. If true, VOP is the VOP
 ;;; associated with this location, for use in determining whether TNs
 ;;; are spilled.
-(defun dump-1-location (node block kind tlf-num label live var-locs vop)
+(defun dump-1-location (node block kind label live var-locs vop
+                        &optional context)
   (declare (type node node) (type ir2-block block)
            (type (or null local-tn-bit-vector) live)
            (type (or label index) label)
-           (type location-kind kind) (type (or index null) tlf-num)
+           (type location-kind kind)
            (type hash-table var-locs) (type (or vop null) vop))
 
-  (let ((byte-buffer *byte-buffer*))
+  (let* ((byte-buffer *byte-buffer*)
+         (stepping (and (combination-p node)
+                        (combination-step-info node)))
+         (live (and live
+                    (compute-live-vars live node block var-locs vop)))
+         (anything-alive (and live
+                              (find 1 live)))
+         (path (node-source-path node))
+         (loc (if (fixnump label) label (label-position label))))
     (vector-push-extend
-     (position-or-lose kind *compiled-code-location-kinds*)
+     (logior
+      (if context
+          compiled-code-location-context
+          0)
+      (if stepping
+          compiled-code-location-stepping
+          0)
+      (if anything-alive
+          compiled-code-location-live
+          0)
+      (if (zerop (source-path-form-number path))
+          compiled-code-location-zero-form-number
+          0)
+      (position-or-lose kind +compiled-code-location-kinds+))
      byte-buffer)
 
-    (let ((loc (if (fixnump label) label (label-position label))))
-      (write-var-integer (- loc *previous-location*) byte-buffer)
-      (setq *previous-location* loc))
+    (write-var-integer (- loc *previous-location*) byte-buffer)
+    (setq *previous-location* loc)
 
-    (let ((path (node-source-path node)))
-      (unless tlf-num
-        (write-var-integer (source-path-tlf-number path) byte-buffer))
+    (unless (zerop (source-path-form-number path))
       (write-var-integer (source-path-form-number path) byte-buffer))
 
-    (if live
-        (write-packed-bit-vector (compute-live-vars live node block var-locs vop)
-                                 byte-buffer)
-        (write-packed-bit-vector
-         (make-array (logandc2 (+ (hash-table-count var-locs) 7) 7)
-                     :initial-element 0
-                     :element-type 'bit)
-         byte-buffer))
-
-    (write-var-string (or (and (typep node 'combination)
-                               (combination-step-info node))
-                          "")
-                      byte-buffer))
+    (when anything-alive
+      (write-packed-bit-vector live byte-buffer))
+    (when stepping
+      (write-var-string stepping byte-buffer))
+    (when context
+      (let ((context (encode-restart-location loc context)))
+        (write-var-integer (or (position context *contexts* :test #'equal)
+                               (vector-push-extend context *contexts*))
+                           byte-buffer))))
   (values))
 
 ;;; Extract context info from a Location-Info structure and use it to
 ;;; dump a compiled code-location.
-(defun dump-location-from-info (loc tlf-num var-locs)
-  (declare (type location-info loc) (type (or index null) tlf-num)
+(defun dump-location-from-info (loc var-locs)
+  (declare (type location-info loc)
            (type hash-table var-locs))
   (let ((vop (location-info-vop loc)))
     (dump-1-location (vop-node vop)
                      (vop-block vop)
                      (location-info-kind loc)
-                     tlf-num
                      (location-info-label loc)
                      (vop-save-set vop)
                      var-locs
-                     vop))
+                     vop
+                     (location-info-context loc)))
   (values))
 
-;;; Scan all the blocks, determining if all locations are in the same
-;;; TLF, and returning it or NIL.
-(defun find-tlf-number (fun)
-  (declare (type clambda fun))
-  (let* ((source-path (node-source-path (lambda-bind fun)))
-         (res (source-path-tlf-number source-path)))
-    (declare (type (or index null) res))
-    (do-physenv-ir2-blocks (2block (lambda-physenv fun))
-      (let ((block (ir2-block-block 2block)))
-        (when (eq (block-info block) 2block)
-          (unless (eql (source-path-tlf-number
-                        (node-source-path
-                         (block-start-node block)))
-                       res)
-            (setq res nil)))
-
-        (dolist (loc (ir2-block-locations 2block))
-          (unless (eql (source-path-tlf-number
-                        (node-source-path
-                         (vop-node (location-info-vop loc))))
-                       res)
-            (setq res nil)))))
-    (values res (source-path-form-number source-path))))
-
 ;;; Dump out the number of locations and the locations for Block.
-(defun dump-block-locations (block locations tlf-num var-locs)
+(defun dump-block-locations (block locations var-locs)
   (declare (type cblock block) (list locations))
-  (if (and locations
-           (eq (location-info-kind (first locations))
-               :non-local-entry))
-      (write-var-integer (length locations) *byte-buffer*)
+  (unless (and locations
+               (eq (location-info-kind (first locations))
+                   :non-local-entry))
       (let ((2block (block-info block)))
-        (write-var-integer (+ (length locations) 1) *byte-buffer*)
         (dump-1-location (block-start-node block)
-                         2block :block-start tlf-num
+                         2block :block-start
                          (ir2-block-%label 2block)
                          (ir2-block-live-out 2block)
                          var-locs
                          nil)))
   (dolist (loc locations)
-    (dump-location-from-info loc tlf-num var-locs))
+    (dump-location-from-info loc var-locs))
   (values))
 
 ;;; Return a vector and an integer (or null) suitable for use as the
 ;;; BLOCKS and TLF-NUMBER in FUN's DEBUG-FUN.
 (defun compute-debug-blocks (fun var-locs)
   (declare (type clambda fun) (type hash-table var-locs))
-  (multiple-value-bind (tlf-num form-number) (find-tlf-number fun)
-   (let ((*previous-location* 0)
-         (physenv (lambda-physenv fun))
-         (byte-buffer *byte-buffer*)
-         prev-block
-         locations
-         elsewhere-locations)
-     (setf (fill-pointer byte-buffer) 0)
-     (do-physenv-ir2-blocks (2block physenv)
-       (let ((block (ir2-block-block 2block)))
-         (when (eq (block-info block) 2block)
-           (when prev-block
-             (dump-block-locations prev-block (nreverse (shiftf locations nil))
-                                   tlf-num var-locs))
-           (setf prev-block block)))
-       (dolist (loc (ir2-block-locations 2block))
-         (if (label-elsewhere-p (location-info-label loc)
-                                (location-info-kind loc))
-             (push loc elsewhere-locations)
-             (push loc locations))))
+  (let ((*previous-location* 0)
+        (physenv (lambda-physenv fun))
+        (byte-buffer *byte-buffer*)
+        prev-block
+        locations
+        elsewhere-locations)
+    (setf (fill-pointer byte-buffer) 0)
+    (do-physenv-ir2-blocks (2block physenv)
+      (let ((block (ir2-block-block 2block)))
+        (when (eq (block-info block) 2block)
+          (when prev-block
+            (dump-block-locations prev-block (nreverse (shiftf locations nil))
+                                  var-locs))
+          (setf prev-block block)))
+      (dolist (loc (ir2-block-locations 2block))
+        (if (label-elsewhere-p (location-info-label loc)
+                               (location-info-kind loc))
+            (push loc elsewhere-locations)
+            (push loc locations))))
 
-     (dump-block-locations prev-block (nreverse locations)
-                           tlf-num var-locs)
+    (dump-block-locations prev-block (nreverse locations) var-locs)
 
-     (when elsewhere-locations
-       (write-var-integer (length elsewhere-locations) byte-buffer)
-       (dolist (loc (nreverse elsewhere-locations))
-         (push loc locations)
-         (dump-location-from-info loc tlf-num var-locs)))
-
-     (values (!make-specialized-array (length byte-buffer) '(unsigned-byte 8)
-                                      byte-buffer)
-             tlf-num form-number))))
+    (when elsewhere-locations
+      (dolist (loc (nreverse elsewhere-locations))
+        (dump-location-from-info loc var-locs)))
+    (let ((compressed
+           (lz-compress (coerce byte-buffer
+                                '(simple-array (unsigned-byte 8) (*))))))
+      (logically-readonlyize
+       (!make-specialized-array (length compressed) '(unsigned-byte 8)
+                                compressed)))))
 
 ;;; Return DEBUG-SOURCE structure containing information derived from
 ;;; INFO.
@@ -253,46 +272,74 @@
 
      :namestring (or *source-namestring*
                      (make-file-info-namestring
-                      (if (pathnamep (file-info-name file-info))
-                          (file-info-name file-info))
+                      (let ((pathname
+                             (case *name-context-file-path-selector*
+                               (pathname (file-info-untruename file-info))
+                               (truename (file-info-name file-info)))))
+                        (if (pathnamep pathname) pathname))
                       file-info))
      :created (file-info-write-date file-info)
-     :start-positions (coerce-to-smallest-eltype
-                       (file-info-positions file-info))
-
-     :form (let ((direct-file-info (source-info-file-info info)))
-             (when (eq :lisp (file-info-name direct-file-info))
-               (elt (file-info-forms direct-file-info) 0)))
+     :form (when function
+             (let ((direct-file-info (source-info-file-info info)))
+               (when (eq :lisp (file-info-name direct-file-info))
+                 (elt (file-info-forms direct-file-info) 0))))
      :function function)))
 
+(defun smallest-element-type (integer negative)
+  (let ((bits (max (+ (integer-length integer)
+                      (if negative 1 0))
+                   8)))
+    (list (if negative
+              'signed-byte
+              'unsigned-byte)
+          (if (= (logcount bits) 1) ;; power of two?
+              bits
+              ;; Next power of two
+              (ash 1 (integer-length bits))))))
+
 ;;; Given an arbitrary sequence, coerce it to an unsigned vector if
 ;;; possible. Ordinarily we coerce it to the smallest specialized
 ;;; vector we can.
 ;;; During cross-compilation the in-memory representation is opaque -
 ;;; we don't care how it looks, but can recover the intended specialization.
-
 (defun coerce-to-smallest-eltype (seq)
-  (let ((maxoid 0) (length 0))
+  (let ((max-positive 0)
+        (max-negative 0)
+        (length 0))
     (flet ((frob (x)
-             (if (typep x 'unsigned-byte)
-                 (when (>= x maxoid)
-                   (setf maxoid x))
-                 (return-from coerce-to-smallest-eltype
-                   (coerce seq 'simple-vector)))))
+             (typecase x
+               ((integer 0)
+                (when (>= x max-positive)
+                  (setf max-positive x)))
+               ((integer * -1)
+                (let ((abs (- x)))
+                  (when (>= abs max-negative)
+                    (setf max-negative abs))))
+               (t
+                (return-from coerce-to-smallest-eltype
+                  (logically-readonlyize
+                   (coerce seq 'simple-vector)))))))
       (if (listp seq)
           (dolist (i seq)
-            (incf length) ; so not to traverse again to compute it
+            (incf length)     ; so not to traverse again to compute it
             (frob i))
           (dovector (i seq (setq length (length seq)))
             (frob i)))
-      (let ((specializer (etypecase maxoid
-                           ((unsigned-byte 8) '(unsigned-byte 8))
-                           ((unsigned-byte 16) '(unsigned-byte 16))
-                           ((unsigned-byte 32) '(unsigned-byte 32))
-                           ((unsigned-byte 64) '(unsigned-byte 64)))))
-        ;; formerly (coerce seq `(simple-array ,specializer (*)))
-        ;; plus a kludge for cross-compilation. This is nicer.
-        (!make-specialized-array length specializer seq)))))
+      (if (zerop length)
+          #()
+          (logically-readonlyize
+           (!make-specialized-array length
+                                    (smallest-element-type (max max-positive
+                                                                (1- max-negative))
+                                                           (plusp max-negative))
+                                    seq))))))
+
+(defun compact-vector (sequence)
+  (cond ((and (= (length sequence) 1)
+              (not (vectorp (elt sequence 0))))
+         (elt sequence 0))
+        (t
+         (coerce-to-smallest-eltype sequence))))
 
 ;;;; variables
 
@@ -321,8 +368,8 @@
 ;;; environment live and is an argument. If a :DEBUG-ENVIRONMENT TN,
 ;;; then we also exclude set variables, since the variable is not
 ;;; guaranteed to be live everywhere in that case.
-(defun dump-1-var (fun var tn id minimal buffer)
-  (declare (type lambda-var var) (type (or tn null) tn) (type index id)
+(defun dump-1-var (fun var tn minimal buffer &optional same-name-p)
+  (declare (type lambda-var var) (type (or tn null) tn)
            (type clambda fun))
   (let* ((name (leaf-debug-name var))
          (save-tn (and tn (tn-save-tn tn)))
@@ -332,7 +379,8 @@
          (indirect (and (lambda-var-indirect var)
                         (not (lambda-var-explicit-value-cell var))
                         (neq (lambda-physenv fun)
-                             (lambda-physenv (lambda-var-home var))))))
+                             (lambda-physenv (lambda-var-home var)))))
+         more)
     (declare (type index flags))
     (when minimal
       (setq flags (logior flags compiled-debug-var-minimal-p))
@@ -347,17 +395,21 @@
       (setq flags (logior flags compiled-debug-var-environment-live)))
     (when save-tn
       (setq flags (logior flags compiled-debug-var-save-loc-p)))
-    (unless (or (zerop id) minimal)
-      (setq flags (logior flags compiled-debug-var-id-p)))
     (when indirect
       (setq flags (logior flags compiled-debug-var-indirect-p)))
     (when info
       (case (arg-info-kind info)
         (:more-context
-         (setq flags (logior flags compiled-debug-var-more-context-p)))
+         (setq flags (logior flags compiled-debug-var-more-context-p)
+               more t))
         (:more-count
-         (setq flags (logior flags compiled-debug-var-more-count-p)))))
-    #!+64-bit
+         (setq flags (logior flags compiled-debug-var-more-count-p)
+               more t))))
+    (when (and same-name-p
+               (not (or more minimal)))
+      (setf flags (logior flags compiled-debug-var-same-name-p)))
+    #!+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
+              ; which early-vm.lisp claims to work
     (cond (indirect
            (setf (ldb (byte 27 8) flags) (tn-sc-offset tn))
            (when save-tn
@@ -369,10 +421,16 @@
            (when save-tn
              (setf (ldb (byte 27 35) flags) (tn-sc-offset save-tn)))))
     (vector-push-extend flags buffer)
-    (unless minimal
-      (vector-push-extend name buffer)
-      (unless (zerop id)
-        (vector-push-extend id buffer)))
+    (unless (or minimal
+                same-name-p
+                more) ;; &more vars need no name
+      ;; Dumping uninterned symbols as debug var names is kinda silly.
+      ;; Reconstruction of the name on fasl load produces a new gensym anyway.
+      ;; So rather than waste symbol space, just dump such symbols as strings,
+      ;; and PARSE-COMPILED-DEBUG-VARS can create the interned symbol.
+      ;; This reduces core size by omitting zillions of symbols whose names
+      ;; are spelled the same.
+      (vector-push-extend (if (symbol-package name) name (string name)) buffer))
 
     (cond (indirect
            ;; Indirect variables live in the parent frame, and are
@@ -411,35 +469,33 @@
                (dolist (leaf (lambda-vars x))
                  (frob-leaf leaf (leaf-info leaf) gensym-p))))
       (frob-lambda fun t)
-      (when (>= level 2)
+      (when (>= level 1)
         (dolist (x (ir2-physenv-closure (physenv-info (lambda-physenv fun))))
           (let ((thing (car x)))
             (when (lambda-var-p thing)
-              (frob-leaf thing (cdr x) (= level 3)))))
+              (frob-leaf thing (cdr x) (>= level 2)))))
 
         (dolist (let (lambda-lets fun))
-          (frob-lambda let (= level 3)))))
+          (frob-lambda let (>= level 2)))))
 
     (let ((sorted (sort (vars) #'string<
                         :key (lambda (x)
                                (symbol-name (leaf-debug-name (car x))))))
           (prev-name nil)
-          (id 0)
           (i 0)
-          (buffer (make-array 0 :fill-pointer 0 :adjustable t)))
-      (declare (type (or simple-string null) prev-name)
-               (type index id i))
+          (buffer (make-array 0 :fill-pointer 0 :adjustable t))
+          ;; XEPs don't have any useful variables
+          (minimal (eq (functional-kind fun) :external)))
+      (declare (type index i))
       (dolist (x sorted)
         (let* ((var (car x))
-               (name (symbol-name (leaf-debug-name var))))
-          (cond ((and prev-name (string= prev-name name))
-                 (incf id))
-                (t
-                 (setq id 0  prev-name name)))
-          (dump-1-var fun var (cdr x) id nil buffer)
+               (name (leaf-debug-name var)))
+          (dump-1-var fun var (cdr x) minimal buffer
+                      (and prev-name (eq prev-name name)))
+          (setf prev-name name)
           (setf (gethash var var-locs) i)
           (incf i)))
-      (coerce buffer 'simple-vector))))
+      (compact-vector buffer))))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS of
 ;;; FUN, representing the arguments to FUN in minimal variable format.
@@ -447,11 +503,11 @@
   (declare (type clambda fun))
   (let ((buffer (make-array 0 :fill-pointer 0 :adjustable t)))
     (dolist (var (lambda-vars fun))
-      (dump-1-var fun var (leaf-info var) 0 t buffer))
-    (coerce buffer 'simple-vector)))
+      (dump-1-var fun var (leaf-info var) t buffer))
+    (compact-vector buffer)))
 
 ;;; Return VAR's relative position in the function's variables (determined
-;;; from the VAR-LOCS hashtable).  If VAR is deleted, then return DELETED.
+;;; from the VAR-LOCS hashtable).  If VAR is deleted, then return DEBUG-INFO-VAR-DELETED.
 (defun debug-location-for (var var-locs)
   (declare (type lambda-var var) (type hash-table var-locs))
   (let ((res (gethash var var-locs)))
@@ -459,7 +515,7 @@
           (t
            (aver (or (null (leaf-refs var))
                      (not (tn-offset (leaf-info var)))))
-           'deleted))))
+           debug-info-var-deleted))))
 
 ;;;; arguments/returns
 
@@ -492,16 +548,16 @@
                                             (return-from one-arg))
                                            (more
                                             (setf (arg-info-default info) t)))
-                                     (res 'rest-arg)))
+                                     (res debug-info-var-rest)))
                                   (:more-context
-                                   (res 'more-arg))
+                                   (res debug-info-var-more))
                                   (:optional
                                    (unless saw-optional
-                                     (res 'optional-args)
+                                     (res debug-info-var-optional)
                                      (setq saw-optional t))))
                                 (res (debug-location-for actual var-locs))
                                 (when (arg-info-supplied-p info)
-                                  (res 'supplied-p)
+                                  (res debug-info-var-supplied-p)
                                   (res (debug-location-for (pop actual-vars) var-locs))))
                                 (t
                                  (res (debug-location-for actual var-locs)))))))
@@ -510,14 +566,13 @@
           (dolist (var (lambda-vars fun))
             (res (debug-location-for var var-locs)))))
 
-    (coerce-to-smallest-eltype (res))))
+    (compact-vector (res))))
 
 ;;; Return a vector of SC offsets describing FUN's return locations.
 ;;; (Must be known values return...)
 (defun compute-debug-returns (fun)
   (coerce-to-smallest-eltype
-   (mapcar (lambda (loc)
-             (tn-sc-offset loc))
+   (mapcar #'tn-sc-offset
            (return-info-locations (tail-set-info (lambda-tail-set fun))))))
 
 ;;;; debug functions
@@ -528,23 +583,38 @@
   (let* ((2env (physenv-info (lambda-physenv fun)))
          (dispatch (lambda-optional-dispatch fun))
          (main-p (and dispatch
-                      (eq fun (optional-dispatch-main-entry dispatch)))))
-    (make-compiled-debug-fun
-     :name (leaf-debug-name fun)
-     :kind (if main-p nil (functional-kind fun))
-     #!-fp-and-pc-standard-save :return-pc
-     #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-return-pc 2env))
-     #!-fp-and-pc-standard-save :old-fp
-     #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-old-fp 2env))
-     :start-pc (label-position (ir2-physenv-environment-start 2env))
-     :elsewhere-pc (label-position (ir2-physenv-elsewhere-start 2env))
-     :closure-save (when (ir2-physenv-closure-save-tn 2env)
-                     (tn-sc-offset (ir2-physenv-closure-save-tn 2env)))
-     #!+unwind-to-frame-and-call-vop
-     :bsp-save
-     #!+unwind-to-frame-and-call-vop
-     (when (ir2-physenv-bsp-save-tn 2env)
-       (tn-sc-offset (ir2-physenv-bsp-save-tn 2env))))))
+                      (eq fun (optional-dispatch-main-entry dispatch))))
+         (kind (if main-p nil (functional-kind fun)))
+         (name (leaf-debug-name fun))
+         (name (if (consp name)
+                   (case (car name)
+                     ((xep tl-xep)
+                      (assert (eq kind :external))
+                      (second name))
+                     (&optional-processor
+                      (setf kind :optional)
+                      (second name))
+                     (&more-processor
+                      (setf kind :more)
+                      (second name))
+                     (t
+                      name))
+                   name)))
+    (funcall (compiled-debug-fun-ctor kind)
+             :name name
+             #!-fp-and-pc-standard-save :return-pc
+             #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-return-pc 2env))
+             #!-fp-and-pc-standard-save :old-fp
+             #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-old-fp 2env))
+             :encoded-locs
+             (cdf-encode-locs
+              (label-position (ir2-physenv-environment-start 2env))
+              (label-position (ir2-physenv-elsewhere-start 2env))
+              (when (ir2-physenv-closure-save-tn 2env)
+                (tn-sc-offset (ir2-physenv-closure-save-tn 2env)))
+              #!+unwind-to-frame-and-call-vop
+              (when (ir2-physenv-bsp-save-tn 2env)
+                (tn-sc-offset (ir2-physenv-bsp-save-tn 2env)))))))
 
 ;;; Return a complete C-D-F structure for FUN. This involves
 ;;; determining the DEBUG-INFO level and filling in optional slots as
@@ -553,16 +623,15 @@
   (declare (type clambda fun) (type hash-table var-locs))
   (let* ((dfun (dfun-from-fun fun))
          (actual-level (policy (lambda-bind fun) compute-debug-fun))
-         (level (if #!+sb-dyncount *collect-dynamic-statistics*
-                    #!-sb-dyncount nil
-                    (max actual-level 2)
-                    actual-level))
-         (toplevel-p (eq :toplevel (compiled-debug-fun-kind dfun))))
-    (cond ((or (zerop level) toplevel-p))
-          ((and (<= level 1)
-                (let ((od (lambda-optional-dispatch fun)))
-                  (or (not od)
-                      (not (eq (optional-dispatch-main-entry od) fun)))))
+         (level (cond #!+sb-dyncount
+                      (*collect-dynamic-statistics*
+                       (max actual-level 2))
+                      (actual-level))))
+    (cond ((or (and (zerop level)
+                    (let ((od (lambda-optional-dispatch fun)))
+                      (or (not od)
+                          (not (eq (optional-dispatch-main-entry od) fun)))))
+               (eq (compiled-debug-fun-kind dfun) :more))
            (setf (compiled-debug-fun-vars dfun)
                  (compute-minimal-vars fun))
            (setf (compiled-debug-fun-arguments dfun) :minimal))
@@ -571,16 +640,11 @@
                  (compute-vars fun level var-locs))
            (setf (compiled-debug-fun-arguments dfun)
                  (compute-args fun var-locs))))
-
-    (if (and (>= level 2) (not toplevel-p))
-        (multiple-value-bind (blocks tlf-num form-number)
-            (compute-debug-blocks fun var-locs)
-          (setf (compiled-debug-fun-blocks dfun) blocks
-                (compiled-debug-fun-tlf-number dfun) tlf-num
-                (compiled-debug-fun-form-number dfun) form-number))
-        (multiple-value-bind (tlf-num form-number) (find-tlf-number fun)
-          (setf (compiled-debug-fun-tlf-number dfun) tlf-num
-                (compiled-debug-fun-form-number dfun) form-number)))
+    (setf (compiled-debug-fun-form-number dfun)
+          (source-path-form-number (node-source-path (lambda-bind fun))))
+    (when (>= level 1)
+      (setf (compiled-debug-fun-blocks dfun)
+            (compute-debug-blocks fun var-locs)))
     (if (xep-p fun)
         (setf (compiled-debug-fun-returns dfun) :standard)
         (let ((info (tail-set-info (lambda-tail-set fun))))
@@ -619,16 +683,41 @@
         (*byte-buffer* (make-array 10
                                    :element-type '(unsigned-byte 8)
                                    :fill-pointer 0
-                                   :adjustable t)))
+                                   :adjustable t))
+        (*contexts* (make-array 10
+                                :fill-pointer 0
+                                :adjustable t))
+        component-tlf-num)
     (dolist (lambda (component-lambdas component))
       (clrhash var-locs)
-      (push (cons (label-position (block-label (lambda-block lambda)))
-                  (compute-1-debug-fun lambda var-locs))
-            dfuns))
+      (let ((tlf-num (source-path-tlf-number
+                      (node-source-path (lambda-bind lambda)))))
+        (if component-tlf-num
+            (aver (= component-tlf-num tlf-num))
+            (setf component-tlf-num tlf-num))
+        (push (cons (label-position (block-label (lambda-block lambda)))
+                    (compute-1-debug-fun lambda var-locs))
+              dfuns)))
     (let* ((sorted (sort dfuns #'< :key #'car))
            (fun-map (compute-debug-fun-map sorted)))
-      (make-compiled-debug-info :name (component-name component)
-                                :fun-map fun-map))))
+      (make-compiled-debug-info
+       ;; COMPONENT-NAME is often not useful, and sometimes completely fubar.
+       ;; Function names, on the other hand, are seldom unhelpful,
+       ;; so if there's only one function, pick that as the component name.
+       ;; Otherwise preserve whatever crummy name was already assigned.
+       :name (let* ((2comp (component-info component))
+                    (entries (sb!c::ir2-component-entries 2comp)))
+               (or (and (not (cdr entries))
+                        (sb!c::entry-info-name (car entries)))
+                   (component-name component)))
+       :fun-map fun-map
+       :tlf-number component-tlf-num
+       :char-offset
+       (and component-tlf-num
+            (aref (file-info-positions
+                   (source-info-file-info *source-info*))
+                  component-tlf-num))
+       :contexts (compact-vector *contexts*)))))
 
 ;;; Write BITS out to BYTE-BUFFER in backend byte order. The length of
 ;;; BITS must be evenly divisible by eight.
