@@ -870,11 +870,14 @@
 (defun ltn-analyze-cast (cast)
   (declare (type cast cast))
   (setf (node-tail-p cast) nil)
-  (when (and (cast-type-check cast)
-             (not (node-lvar cast)))
-    ;; FIXME
-    (bug "IR2 type checking of unused values is not implemented.")
-    )
+  (when (not (node-lvar cast))
+    (aver (not (cast-type-check cast)))
+    (labels ((unlink (lvar)
+               (do-uses (node lvar)
+                 (if (cast-p node)
+                     (unlink (cast-value cast))
+                     (setf (node-lvar node) nil)))))
+      (unlink (cast-value cast))))
   (values))
 
 (defun ltn-annotate-casts (lvar)
