@@ -46,6 +46,10 @@
   (values))
 
 ;;;; Computation of live UVL sets
+(defun nle-block-p (block)
+  (and (eq (component-head (block-component block))
+           (first (block-pred block)))
+       (not (bind-p (block-start-node block)))))
 (defun nle-block-nlx-info (block)
   (let* ((start-node (block-start-node block))
          (nlx-ref (ctran-next (node-next start-node)))
@@ -93,13 +97,30 @@
   ;; over an arbitrarily complex chunk of flow graph that is known to
   ;; have a single entry block.
   (let* ((use-blocks (mapcar #'node-block (find-uses dx-lvar)))
+         ;; We have to back-propagate not just the DX-LVAR, but every
+         ;; UVL or DX LVAR that is live wherever DX-LVAR is USEd
+         ;; (allocated) because we can't move live DX-LVARs to release
+         ;; them.
+         (preserve-lvars (reduce #'merge-uvl-live-sets
+                                 (mapcar (lambda (block)
+                                           (let ((2block (block-info block)))
+                                             (merge-uvl-live-sets
+                                              (ir2-block-end-stack 2block)
+                                              (ir2-block-pushed 2block))))
+                                         use-blocks)))
          (start-block (find-lowest-common-dominator
                        (list* block use-blocks))))
     (labels ((mark-lvar-live-on-path (arc-list)
                (dolist (arc arc-list)
                  (let ((2block (block-info (car arc))))
-                   (pushnew dx-lvar (ir2-block-end-stack 2block))
-                   (pushnew dx-lvar (ir2-block-start-stack 2block)))))
+                   (setf (ir2-block-end-stack 2block)
+                         (merge-uvl-live-sets
+                          preserve-lvars
+                          (ir2-block-end-stack 2block)))
+                   (setf (ir2-block-start-stack 2block)
+                         (merge-uvl-live-sets
+                          preserve-lvars
+                          (ir2-block-start-stack 2block))))))
              (back-propagate-pathwise (current-block path)
                (cond
                  ((member current-block use-blocks)
@@ -110,7 +131,14 @@
                   (mark-lvar-live-on-path path))
                  ;; Don't go back past START-BLOCK.
                  ((not (eq current-block start-block))
-                  (dolist (pred-block (block-pred current-block))
+                  (dolist (pred-block (if (nle-block-p current-block)
+                                          ;; Follow backwards through
+                                          ;; NLEs to the start of
+                                          ;; their environment
+                                          (list* (nle-block-entry-block
+                                                  current-block)
+                                                 (block-pred current-block))
+                                          (block-pred current-block)))
                     (let ((new-arc (cons current-block pred-block)))
                       (declare (dynamic-extent new-arc))
                       ;; Never follow the same path segment twice.
@@ -185,8 +213,6 @@
     ;; environment) then we need to back-propagate the DX LVARs to
     ;; their allocation sites.  We need to be clever about this
     ;; because some code paths may not allocate all of the DX LVARs.
-    ;;
-    ;; FIXME: Use BLOCK-FLAG to make this happen only once.
     (let ((first-node (ctran-next (block-start block))))
       (when (typep first-node 'entry)
         (let ((cleanup (entry-cleanup first-node)))
@@ -208,9 +234,7 @@
       ;;
       ;; TODO: Insert a check that no values are discarded in UWP. Or,
       ;; maybe, we just don't need to create NLX-ENTRY for UWP?
-      (when (and (eq (component-head (block-component block))
-                     (first (block-pred block)))
-                 (not (bind-p (block-start-node block))))
+      (when (nle-block-p block)
         (let* ((nlx-info (nle-block-nlx-info block))
                (cleanup (nlx-info-cleanup nlx-info)))
           (unless (eq (cleanup-kind cleanup) :unwind-protect)
