@@ -46,7 +46,6 @@
 
 #include "validate.h"
 #include "gc-internal.h"
-#include "runtime-options.h"
 #include "pseudo-atomic.h"
 
 #include <errno.h>
@@ -70,7 +69,7 @@ unsigned char build_id[] =
 #endif
 ;
 
-int
+static int
 open_binary(char *filename, int mode)
 {
 #ifdef LISP_FEATURE_WIN32
@@ -80,44 +79,12 @@ open_binary(char *filename, int mode)
     return open(filename, mode);
 }
 
-
-static struct runtime_options *
-read_runtime_options(int fd)
-{
-    os_vm_size_t optarray[RUNTIME_OPTIONS_WORDS];
-    struct runtime_options *options = NULL;
-
-    if (read(fd, optarray, RUNTIME_OPTIONS_WORDS * sizeof(os_vm_size_t)) !=
-        RUNTIME_OPTIONS_WORDS * sizeof(size_t)) {
-        return NULL;
-    }
-
-    if ((RUNTIME_OPTIONS_MAGIC != optarray[0]) || (0 == optarray[1])) {
-        return NULL;
-    }
-
-    options = successful_malloc(sizeof(struct runtime_options));
-
-    options->dynamic_space_size = optarray[2];
-    options->thread_control_stack_size = optarray[3];
-
-    return options;
-}
-
-void
-maybe_initialize_runtime_options(int fd)
-{
-    struct runtime_options *new_runtime_options;
-    off_t end_offset = sizeof(lispobj) +
-        sizeof(os_vm_offset_t) +
-        (RUNTIME_OPTIONS_WORDS * sizeof(size_t));
-
-    lseek(fd, -end_offset, SEEK_END);
-
-    if ((new_runtime_options = read_runtime_options(fd))) {
-        runtime_options = new_runtime_options;
-    }
-}
+#if defined(LISP_FEATURE_LINUX) && defined(LISP_FEATURE_IMMOBILE_CODE)
+#define ELFCORE 1
+extern __attribute__((weak)) lispobj __lisp_code_start, __lisp_code_end;
+#else
+#define ELFCORE 0
+#endif
 
 /* Search 'filename' for an embedded core.  An SBCL core has, at the
  * end of the file, a trailer containing optional saved runtime
@@ -129,58 +96,59 @@ maybe_initialize_runtime_options(int fd)
  * If an embedded core is present, this returns the offset into the
  * file to load the core from, or -1 if no core is present. */
 os_vm_offset_t
-search_for_embedded_core(char *filename)
+search_for_embedded_core(char *filename, struct memsize_options *memsize_options)
 {
-    lispobj header;
+    extern os_vm_offset_t search_for_elf_core(int);
+    lispobj header = 0;
     os_vm_offset_t lispobj_size = sizeof(lispobj);
-    os_vm_offset_t trailer_size = lispobj_size + sizeof(os_vm_offset_t);
-    os_vm_offset_t core_start, pos;
-    int fd = -1;
+    int fd;
 
     if ((fd = open_binary(filename, O_RDONLY)) < 0)
-        goto lose;
+        return -1;
 
-    if (read(fd, &header, (size_t)lispobj_size) < lispobj_size)
-        goto lose;
-    if (header == CORE_MAGIC) {
+    if (read(fd, &header, lispobj_size) == lispobj_size && header == CORE_MAGIC) {
         /* This file is a real core, not an embedded core.  Return 0 to
          * indicate where the core starts, and do not look for runtime
          * options in this case. */
+        close(fd);
         return 0;
     }
 
-    if (lseek(fd, -lispobj_size, SEEK_END) < 0)
-        goto lose;
-    if (read(fd, &header, (size_t)lispobj_size) < lispobj_size)
+    os_vm_offset_t core_start = -1; // invalid value
+    if (lseek(fd, -lispobj_size, SEEK_END) < 0 ||
+        read(fd, &header, (size_t)lispobj_size) != lispobj_size)
         goto lose;
 
     if (header == CORE_MAGIC) {
-        if (lseek(fd, -trailer_size, SEEK_END) < 0)
+        // the last word in the file could be CORE_MAGIC by pure coincidence
+        if (lseek(fd, -(lispobj_size + sizeof(os_vm_offset_t)), SEEK_END) < 0 ||
+            read(fd, &core_start, sizeof(os_vm_offset_t)) != sizeof(os_vm_offset_t))
             goto lose;
-        if (read(fd, &core_start, sizeof(os_vm_offset_t)) < 0)
-            goto lose;
-
-        if (lseek(fd, core_start, SEEK_SET) < 0)
-            goto lose;
-        pos = lseek(fd, 0, SEEK_CUR);
-
-        if (read(fd, &header, (size_t)lispobj_size) < lispobj_size)
-            goto lose;
-
-        if (header != CORE_MAGIC)
-            goto lose;
-
-        maybe_initialize_runtime_options(fd);
-
-        close(fd);
-        return pos;
+        if (lseek(fd, core_start, SEEK_SET) != core_start ||
+            read(fd, &header, lispobj_size) != lispobj_size || header != CORE_MAGIC)
+            core_start = -1; // reset to invalid
+    }
+#if ELFCORE
+    if (core_start < 0) {
+        if (!(core_start = search_for_elf_core(fd)) ||
+            lseek(fd, core_start, SEEK_SET) != core_start ||
+            read(fd, &header, lispobj_size) != lispobj_size || header != CORE_MAGIC)
+            core_start = -1; // reset to invalid
+    }
+#endif
+    if (core_start > 0 && memsize_options) {
+        core_entry_elt_t optarray[RUNTIME_OPTIONS_WORDS];
+        // file is already positioned to the first core header entry
+        if (read(fd, optarray, sizeof optarray) == sizeof optarray
+            && optarray[0] == RUNTIME_OPTIONS_MAGIC) {
+            memsize_options->dynamic_space_size = optarray[2];
+            memsize_options->thread_control_stack_size = optarray[3];
+            memsize_options->present_in_core = 1;
+        }
     }
-
 lose:
-    if (fd != -1)
-        close(fd);
-
-    return -1;
+    close(fd);
+    return core_start;
 }
 
 #ifndef LISP_FEATURE_HPUX
@@ -292,12 +260,16 @@ static void inflate_core_bytes(int fd, os_vm_offset_t offset,
 # undef ZLIB_BUFFER_SIZE
 #endif
 
+#define DYNAMIC_SPACE_ADJ_INDEX 0
 struct heap_adjust {
-    /* range[0] is immobile space, range [1] is dynamic space */
+    /* range[0] is dynamic space, ranges[1] and [2] are immobile spaces */
     struct range {
         lispobj start, end;
         sword_t delta;
-    } range[2];
+    } range[3];
+    int n_ranges;
+    int n_relocs_abs; // absolute
+    int n_relocs_rel; // relative
 };
 
 #ifndef LISP_FEATURE_RELOCATABLE_HEAP
@@ -311,12 +283,14 @@ struct heap_adjust {
 
 static inline sword_t calc_adjustment(struct heap_adjust* adj, lispobj x)
 {
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
     if (adj->range[0].start <= x && x < adj->range[0].end)
         return adj->range[0].delta;
-#endif
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
     if (adj->range[1].start <= x && x < adj->range[1].end)
         return adj->range[1].delta;
+    if (adj->range[2].start <= x && x < adj->range[2].end)
+        return adj->range[2].delta;
+#endif
     return 0;
 }
 
@@ -326,6 +300,33 @@ static inline lispobj adjust_word(struct heap_adjust* adj, lispobj word) {
     return word + calc_adjustment(adj, word);
 }
 
+#define SHOW_SPACE_RELOCATION 0
+#if SHOW_SPACE_RELOCATION > 1
+# define FIXUP(expr, addr) fprintf(stderr, "%p: (a) %lx", addr, *(long*)(addr)), \
+   expr, fprintf(stderr, " -> %lx\n", *(long*)(addr)), ++adj->n_relocs_abs
+# define FIXUP32(expr, addr) fprintf(stderr, "%p: (a) %x", addr, *(int*)(addr)), \
+   expr, fprintf(stderr, " -> %x\n", *(int*)(addr)), ++adj->n_relocs_abs
+# define FIXUP_rel(expr, addr) fprintf(stderr, "%p: (r) %x", addr, *(int*)(addr)), \
+   expr, fprintf(stderr, " -> %x\n", *(int*)(addr)), ++adj->n_relocs_rel
+#elif SHOW_SPACE_RELOCATION
+# define FIXUP(expr, addr) expr, ++adj->n_relocs_abs
+# define FIXUP32(expr, addr) expr, ++adj->n_relocs_abs
+# define FIXUP_rel(expr, addr) expr, ++adj->n_relocs_rel
+#else
+# define FIXUP(expr, addr) expr
+# define FIXUP32(expr, addr) expr
+# define FIXUP_rel(expr, addr) expr
+#endif
+
+// Fix the word at 'where' without testing whether it looks pointer-like.
+// Avoid writing if there is no adjustment.
+static inline void adjust_word_at(lispobj* where, struct heap_adjust* adj) {
+    lispobj word = *where;
+    sword_t adjustment = calc_adjustment(adj, word);
+    if (adjustment != 0)
+        FIXUP(*where = word + adjustment, where);
+}
+
 // Adjust the words in range [where,where+n_words)
 // skipping any words that have non-pointer nature.
 static void adjust_pointers(lispobj *where, sword_t n_words, struct heap_adjust* adj)
@@ -335,7 +336,7 @@ static void adjust_pointers(lispobj *where, sword_t n_words, struct heap_adjust*
         lispobj word = where[i];
         sword_t adjustment;
         if (is_lisp_pointer(word) && (adjustment = calc_adjustment(adj, word)) != 0) {
-            where[i] += adjustment;
+            FIXUP(where[i] = word + adjustment, where+i);
         }
     }
 }
@@ -354,18 +355,23 @@ adjust_code_refs(struct heap_adjust* adj, lispobj fixups, struct code* code)
         // so that the magnitudes are smaller.
         loc += prev_loc;
         prev_loc = loc;
-        int* fixup_where = (int*)(instructions + loc);
+        void* fixup_where = instructions + loc;
         lispobj ptr = UNALIGNED_LOAD32(fixup_where);
-        UNALIGNED_STORE32(fixup_where, ptr + calc_adjustment(adj, ptr));
+        lispobj adjusted = ptr + calc_adjustment(adj, ptr);
+        if (adjusted != ptr)
+            FIXUP32(UNALIGNED_STORE32(fixup_where, adjusted), fixup_where);
     }
 }
 
+static inline void fix_fun_header_layout(lispobj* fun, struct heap_adjust* adj)
+{
 #if defined(LISP_FEATURE_COMPACT_INSTANCE_HEADER) && defined(LISP_FEATURE_64_BIT)
-#define FIX_FUN_HEADER_LAYOUT(fun) \
-  set_function_layout(fun, adjust_word(adj, function_layout(fun)))
-#else
-#define FIX_FUN_HEADER_LAYOUT(f) {}
+    lispobj ptr = function_layout(fun);
+    lispobj adjusted = adjust_word(adj, ptr);
+    if (adjusted != ptr)
+        FIXUP(set_function_layout(fun, adjusted), fun);
 #endif
+}
 
 static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 {
@@ -377,6 +383,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
     struct code* code;
     sword_t delta;
 
+    adj->n_relocs_abs = adj->n_relocs_rel = 0;
     for ( ; where < end ; where += nwords ) {
         header_word = *where;
         if (is_cons_half(header_word)) {
@@ -393,7 +400,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             ///  hence needs no adjustment.
             // - Otherwise, the word might point to a relocated range,
             //   either the instance itself, or a trampoline in immobile space.
-            where[1] = adjust_word(adj, where[1]);
+            adjust_word_at(where+1, adj);
         case INSTANCE_WIDETAG:
             layout = (widetag == FUNCALLABLE_INSTANCE_WIDETAG) ?
                 funinstance_layout(where) : instance_layout(where);
@@ -427,13 +434,15 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             // so adjust_pointers() would ignore it. Therefore we need to
             // forcibly adjust it.
 #ifndef LISP_FEATURE_IMMOBILE_CODE
-            where[3] = adjust_word(adj, where[3]);
+            adjust_word_at(where+3, adj);
 #elif defined(LISP_FEATURE_X86_64)
-            // static space to immobile space JMP needs adjustment
-            if (STATIC_SPACE_START <= (uintptr_t)where && (uintptr_t)where < STATIC_SPACE_END) {
-                delta = calc_adjustment(adj, fdefn_callee_lispobj((struct fdefn*)where));
-                if (delta != 0)
-                    *(int*)(1+(char*)(where+3)) += delta;
+            // the offset from fdefns to code can change if:
+            // - static space fdefn -> immobile space code and immobile space moved
+            // - immobile spaces changed relative position
+            if ((delta = calc_adjustment(adj, fdefn_callee_lispobj((struct fdefn*)where)))) {
+                void* prel32 = 1 + (char*)(where+3);
+                FIXUP_rel(UNALIGNED_STORE32(prel32, UNALIGNED_LOAD32(prel32) + delta),
+                          prel32);
             }
 #endif
             continue;
@@ -443,14 +452,18 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             // Fixup all embedded simple-funs
             code = (struct code*)where;
             for_each_simple_fun(i, f, code, 1, {
-                FIX_FUN_HEADER_LAYOUT((lispobj*)f);
-                f->self = adjust_word(adj, f->self);
+                fix_fun_header_layout((lispobj*)f, adj);
+#if FUN_SELF_FIXNUM_TAGGED
+                if (f->self != (lispobj)f->code)
+                    FIXUP(f->self = (lispobj)f->code, &f->self);
+#endif
                 adjust_pointers(SIMPLE_FUN_SCAV_START(f), SIMPLE_FUN_SCAV_NWORDS(f), adj);
             });
             // Compute the address where the code "was" as the first argument
             // by negating the adjustment for 'where'.
             // Can't call calc_adjustment to get the negative of the adjustment!
-            gencgc_apply_code_fixups((struct code*)((char*)where - adj->range[1].delta),
+            gencgc_apply_code_fixups((struct code*)((char*)where -
+                                                    adj->range[DYNAMIC_SPACE_ADJ_INDEX].delta),
                                      code);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
             // Now that the packed integer comprising the list of fixup locations
@@ -460,13 +473,13 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 #endif
             continue;
         case CLOSURE_WIDETAG:
-            FIX_FUN_HEADER_LAYOUT(where);
+            fix_fun_header_layout(where, adj);
 #if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
             // For x86[-64], the closure fun appears to be a fixnum,
             // and might need adjustment unless pointing to immobile code.
             // Then fall into the general case; where[1] won't get re-adjusted
             // because it doesn't satisfy is_lisp_pointer().
-            where[1] = adjust_word(adj, where[1]);
+            adjust_word_at(where+1, adj);
 #endif
             break;
         // Vectors require extra care because of EQ-based hashing.
@@ -479,17 +492,19 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
               lispobj* data = (lispobj*)v->data;
               adjust_pointers(&data[0], 1, adj); // adjust the hash-table structure
               boolean needs_rehash = 0;
-              int i;
+              lispobj *where = &data[2], *end = &data[fixnum_value(v->length)];
               // Adjust the elements, checking for need to rehash.
-              // v->data[1] is the unbound marker (a non-pointer)
-              for (i = fixnum_value(v->length)-1 ; i>=2 ; --i) {
-                  lispobj ptr = data[i];
+              for ( ; where < end ; where += 2) {
+                  lispobj ptr = *where; // key
                   if (is_lisp_pointer(ptr) && (delta = calc_adjustment(adj, ptr)) != 0) {
-                      data[i] += delta;
+                      FIXUP(*where = ptr + delta, where);
                       needs_rehash = 1;
                   }
+                  ptr = where[1]; // value
+                  if (is_lisp_pointer(ptr) && (delta = calc_adjustment(adj, ptr)) != 0)
+                      FIXUP(where[1] = ptr + delta, where+1);
               }
-              if (needs_rehash)
+              if (needs_rehash) // set v->data[1], the need-to-rehash bit
                   data[1] = make_fixnum(1);
               continue;
           }
@@ -517,7 +532,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
                 fprintf(stderr,
                         "WARNING: SAP at %p -> %p in relocatable core\n",
                         where, (void*)where[1]);
-                where[1] += delta;
+                FIXUP(where[1] += delta, where+1);
             }
             continue;
         case BIGNUM_WIDETAG:
@@ -540,53 +555,61 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
         }
         adjust_pointers(where+1, nwords-1, adj);
     }
+#if SHOW_SPACE_RELOCATION
+    fprintf(stderr, "space @ %p: fixed %d absolute + %d relative pointers\n",
+            (lispobj*)start, adj->n_relocs_abs, adj->n_relocs_rel);
+#endif
 }
 
-#define SHOW_SPACE_RELOCATION 0
-void relocate_heap(struct heap_adjust* adj)
+static void relocate_heap(struct heap_adjust* adj)
 {
-    if (SHOW_SPACE_RELOCATION) {
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-        fprintf(stderr, "Relocating immobile space from [%p:%p] to [%p:%p]\n",
-                (char*)adj->range[0].start,
-                (char*)adj->range[0].end,
-                (char*)FIXEDOBJ_SPACE_START,
-                (char*)FIXEDOBJ_SPACE_START+(adj->range[0].end-adj->range[0].start));
-#endif
-        fprintf(stderr, "Relocating dynamic space from [%p:%p] to [%p:%p]\n",
-                (char*)adj->range[1].start,
-                (char*)adj->range[1].end,
-                (char*)DYNAMIC_SPACE_START,
-                (char*)DYNAMIC_SPACE_START+(adj->range[1].end-adj->range[1].start));
+    if (!lisp_startup_options.noinform || SHOW_SPACE_RELOCATION) {
+        int i;
+        for (i = 0; i < adj->n_ranges; ++i)
+            if (adj->range[i].delta)
+                fprintf(stderr, "NOTE: Relocating [%p:%p] into [%p:%p]\n",
+                        (char*)adj->range[i].start,
+                        (char*)adj->range[i].end,
+                        (char*)adj->range[i].start + adj->range[i].delta,
+                        (char*)adj->range[i].end + adj->range[i].delta);
     }
     relocate_space(STATIC_SPACE_START, static_space_free_pointer, adj);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     relocate_space(FIXEDOBJ_SPACE_START, fixedobj_free_pointer, adj);
-    relocate_space(VARYOBJ_SPACE_START, varyobj_free_pointer, adj);
     SYMBOL(FUNCTION_LAYOUT)->value = \
         adjust_word(adj, SYMBOL(FUNCTION_LAYOUT)->value >> 32) << 32;
 #endif
     relocate_space(DYNAMIC_SPACE_START, (lispobj*)get_alloc_pointer(), adj);
-    lispobj asmroutines = SYMBOL(ASSEMBLER_ROUTINES)->value;
-    if (lowtag_of(asmroutines) == INSTANCE_POINTER_LOWTAG) {
-        /* Adjust the values in SB-FASL::*ASSEMBLER-ROUTINES*.
-         * No need to frob the 'needs_rehash_p' slot on account of this */
-        struct hash_table *ht = (struct hash_table*)native_pointer(asmroutines);
-        struct vector *table = (struct vector*)native_pointer(ht->table);
-        int i;
-        for (i=fixnum_value(table->length)-1; i>=3; i -= 2) {
-            if (fixnump(table->data[i]))
-                table->data[i] = make_fixnum(adjust_word(adj, fixnum_value(table->data[i])));
-        }
-    }
+
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    // Pointers within varyobj space to varyobj space do not need adjustment
+    // so remove any delta before performing the relocation pass on this space.
+#if ELFCORE
+    if (&__lisp_code_start)
+        adj->range[2].delta = 0;
+#endif
+    relocate_space(VARYOBJ_SPACE_START, varyobj_free_pointer, adj);
+#endif
 }
 #endif
 
-int merge_core_pages = -1;
+static void
+set_adjustment(struct heap_adjust* adj,
+               uword_t actual_addr,
+               uword_t desired_addr,
+               uword_t len)
+{
+    int j = adj->n_ranges;
+    gc_assert(j <= 2);
+    adj->range[j].start = (lispobj)desired_addr;
+    adj->range[j].end   = (lispobj)desired_addr + len;
+    adj->range[j].delta = actual_addr - desired_addr;
+    adj->n_ranges = j+1;
+}
 
 static void
 process_directory(int count, struct ndir_entry *entry,
-                  int fd, os_vm_offset_t file_offset,
+                  int fd, os_vm_offset_t file_offset, int merge_core_pages,
                   struct heap_adjust* adj)
 {
     extern void immobile_space_coreparse(uword_t,uword_t);
@@ -608,15 +631,37 @@ process_directory(int count, struct ndir_entry *entry,
         {0, 0, STATIC_SPACE_START, &static_space_free_pointer},
         {0, 0, READ_ONLY_SPACE_START, &read_only_space_free_pointer},
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-        {(FIXEDOBJ_SPACE_SIZE+VARYOBJ_SPACE_SIZE) | 1, 0,
+        {FIXEDOBJ_SPACE_SIZE | 1, 0,
             FIXEDOBJ_SPACE_START, &fixedobj_free_pointer},
         {1, 0, VARYOBJ_SPACE_START, &varyobj_free_pointer}
 #endif
     };
 
+#if ELFCORE
+    if (&__lisp_code_start) {
+        VARYOBJ_SPACE_START = (uword_t)&__lisp_code_start;
+        varyobj_free_pointer = &__lisp_code_end;
+        uword_t aligned_end = ALIGN_UP((uword_t)&__lisp_code_end, IMMOBILE_CARD_BYTES);
+        varyobj_space_size = aligned_end - VARYOBJ_SPACE_START;
+        spaces[IMMOBILE_VARYOBJ_CORE_SPACE_ID].len = varyobj_space_size;
+        gc_assert(varyobj_free_pointer >= (lispobj*)VARYOBJ_SPACE_START);
+#if !ENABLE_PAGE_PROTECTION
+        printf("Lisp code present in executable @ %lx:%lx (freeptr=%p)\n",
+               (uword_t)&__lisp_code_start, aligned_end, varyobj_free_pointer);
+#endif
+        // unprotect the pages
+        os_protect((void*)VARYOBJ_SPACE_START, varyobj_space_size, OS_VM_PROT_ALL);
+    } else
+#endif
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    {
+        spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].desired_size += VARYOBJ_SPACE_SIZE;
+    }
+#endif
+
     for ( ; --count>= 0; ++entry) {
         sword_t id = entry->identifier;
-        uword_t addr = (1024 * entry->address); // multiplier as per core.h
+        uword_t addr = entry->address;
         int compressed = id & DEFLATED_CORE_SPACE_ID_FLAG;
         id -= compressed;
         if (id < 1 || id > MAX_CORE_SPACE_ID)
@@ -633,6 +678,10 @@ process_directory(int count, struct ndir_entry *entry,
         // Enforce address of readonly and static spaces.
         int enforce_address = id != DYNAMIC_CORE_SPACE_ID;
 #endif
+
+        // We'd like to enforce proper alignment of 'addr' but there's
+        // a problem: dynamic space has a stricter requirement (usually 32K)
+        // than code space (4K). So don't assert the alignment.
         if (enforce_address) {
             int fail;
 #ifdef LISP_FEATURE_CHENEYGC
@@ -656,13 +705,13 @@ process_directory(int count, struct ndir_entry *entry,
         }
         spaces[id].base = addr;
         uword_t len = os_vm_page_size * entry->page_count;
-        spaces[id].len = len;
         if (id == DYNAMIC_CORE_SPACE_ID && len > dynamic_space_size) {
             lose("dynamic space too small for core: %luKiB required, %luKiB available.\n",
                  (unsigned long)len >> 10,
                  (unsigned long)dynamic_space_size >> 10);
         }
         if (len != 0) {
+            spaces[id].len = len;
             uword_t __attribute__((unused)) aligned_start;
 #ifdef LISP_FEATURE_RELOCATABLE_HEAP
             // Try to map at address requested by the core file.
@@ -697,11 +746,6 @@ process_directory(int count, struct ndir_entry *entry,
                  * Drop one card to avoid overrunning the allocated space */
                 if (aligned_start > addr) // not card-aligned
                     dynamic_space_size -= GENCGC_CARD_BYTES;
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-                // FIXME: is this invariant still needed?
-                if (addr < FIXEDOBJ_SPACE_START || addr < VARYOBJ_SPACE_START)
-                    lose("Won't map dynamic space below immobile space");
-#endif
                 DYNAMIC_SPACE_START = addr = aligned_start;
                 break;
             }
@@ -724,7 +768,9 @@ process_directory(int count, struct ndir_entry *entry,
         lispobj *free_pointer = (lispobj *) addr + entry->nwords;
         switch (id) {
         default:
-            *spaces[id].pfree_pointer = free_pointer;
+            // varyobj free ptr is already nonzero if Lisp code in executable
+            if (!*spaces[id].pfree_pointer)
+                *spaces[id].pfree_pointer = free_pointer;
             break;
         case DYNAMIC_CORE_SPACE_ID:
 #ifdef LISP_FEATURE_CHENEYGC
@@ -745,20 +791,18 @@ process_directory(int count, struct ndir_entry *entry,
     }
 
 #ifdef LISP_FEATURE_RELOCATABLE_HEAP
+    set_adjustment(adj, DYNAMIC_SPACE_START, // actual
+                   spaces[DYNAMIC_CORE_SPACE_ID].base, // expected
+                   spaces[DYNAMIC_CORE_SPACE_ID].len);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    if (FIXEDOBJ_SPACE_START != spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].base) {
-        adj->range[0].start = spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].base;
-        adj->range[0].end   = adj->range[0].start + FIXEDOBJ_SPACE_SIZE
-            + spaces[IMMOBILE_VARYOBJ_CORE_SPACE_ID].len;
-        adj->range[0].delta = FIXEDOBJ_SPACE_START - adj->range[0].start;
-    }
+    set_adjustment(adj, FIXEDOBJ_SPACE_START, // actual
+                   spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].base, // expected
+                   spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].len);
+    set_adjustment(adj, VARYOBJ_SPACE_START, // actual
+                   spaces[IMMOBILE_VARYOBJ_CORE_SPACE_ID].base, // expected
+                   spaces[IMMOBILE_VARYOBJ_CORE_SPACE_ID].len);
 #endif
-    if (DYNAMIC_SPACE_START != spaces[DYNAMIC_CORE_SPACE_ID].base) {
-        adj->range[1].start = spaces[DYNAMIC_CORE_SPACE_ID].base;
-        adj->range[1].end   = adj->range[1].start + spaces[DYNAMIC_CORE_SPACE_ID].len;
-        adj->range[1].delta = DYNAMIC_SPACE_START - adj->range[1].start;
-    }
-    if (adj->range[0].delta | adj->range[1].delta)
+    if (adj->range[0].delta | adj->range[1].delta | adj->range[2].delta)
         relocate_heap(adj);
 #endif
 
@@ -798,8 +842,21 @@ process_directory(int count, struct ndir_entry *entry,
 #endif
 }
 
+#ifdef LISP_FEATURE_GENCGC
+extern void gc_load_corefile_ptes(core_entry_elt_t, core_entry_elt_t,
+                                  off_t offset, int fd);
+#else
+#define gc_load_corefile_ptes(dummy1,dummy2,dummy3,dummy4)
+#endif
+
+/* 'merge_core_pages': Tri-state flag to determine whether we attempt to mark
+ * pages as targets for virtual memory deduplication via MADV_MERGEABLE.
+ * 1: Yes
+ * 0: No
+ * -1: default, yes for compressed cores, no otherwise.
+ */
 lispobj
-load_core_file(char *file, os_vm_offset_t file_offset)
+load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
 {
     void *header;
     core_entry_elt_t val, *ptr;
@@ -878,7 +935,7 @@ load_core_file(char *file, os_vm_offset_t file_offset)
             SHOW("NEW_DIRECTORY_CORE_ENTRY_TYPE_CODE case");
             process_directory(remaining_len / NDIR_ENTRY_LENGTH,
                               (struct ndir_entry*)ptr, fd, file_offset,
-                              &adj);
+                              merge_core_pages, &adj);
             break;
 
         case INITIAL_FUN_CORE_ENTRY_TYPE_CODE:
@@ -886,37 +943,11 @@ load_core_file(char *file, os_vm_offset_t file_offset)
             initial_function = adjust_word(&adj, (lispobj)*ptr);
             break;
 
-#ifdef LISP_FEATURE_GENCGC
         case PAGE_TABLE_CORE_ENTRY_TYPE_CODE:
-        {
-            extern void gc_allocate_ptes();
-            extern boolean gc_load_corefile_ptes(char data[], ssize_t,
-                                                 page_index_t, page_index_t*);
-            // Allocation of PTEs is delayed 'til now so that calloc() doesn't
-            // consume addresses that would have been taken by a mapped space.
-            gc_allocate_ptes();
-            os_vm_size_t remaining = *ptr;
-            os_vm_size_t fdoffset = (*(ptr+1) + 1) * (os_vm_page_size);
-            page_index_t page = 0, npages;
-            ssize_t bytes_read;
-            char data[8192];
-            // A corefile_pte is 10 bytes for x86-64
-            // Process an integral number of ptes on each read.
-            os_vm_size_t chunksize = sizeof (struct corefile_pte)
-                * (sizeof data / sizeof (struct corefile_pte));
-            lseek(fd, fdoffset + file_offset, SEEK_SET);
-            bytes_read = read(fd, &npages, sizeof npages);
-            gc_assert(bytes_read == sizeof npages);
-            remaining -= sizeof npages;
-            while ((bytes_read = read(fd, data,
-                                      remaining < chunksize ? remaining : chunksize)) > 0
-                   && gc_load_corefile_ptes(data, bytes_read, npages, &page))
-                remaining -= bytes_read;
-
-            gencgc_partial_pickup = 1;
+            gc_load_corefile_ptes(ptr[0], ptr[1],
+                                  file_offset + (ptr[2] + 1) * os_vm_page_size, fd);
             break;
-        }
-#endif
+        case RUNTIME_OPTIONS_MAGIC: break; // already processed
         default:
             lose("unknown core file entry: 0x%"WORD_FMTX"\n", val);
         }
@@ -925,21 +956,26 @@ load_core_file(char *file, os_vm_offset_t file_offset)
 
 #include "genesis/hash-table.h"
 #include "genesis/vector.h"
+#include "genesis/cons.h"
 os_vm_address_t get_asm_routine_by_name(const char* name)
 {
-    lispobj routines = SYMBOL(ASSEMBLER_ROUTINES)->value;
-    if (lowtag_of(routines) == INSTANCE_POINTER_LOWTAG) {
-        struct hash_table* ht = (struct hash_table*)native_pointer(routines);
+#ifdef LISP_FEATURE_IMMOBILE_CODE
+    struct code* code = (struct code*)VARYOBJ_SPACE_START;
+#else
+    struct code* code = (struct code*)READ_ONLY_SPACE_START;
+#endif
+    if (lowtag_of(code->debug_info) == LIST_POINTER_LOWTAG) {
+        struct hash_table* ht =
+            (struct hash_table*)native_pointer(CONS(code->debug_info)->car);
         struct vector* table = VECTOR(ht->table);
         lispobj sym;
         int i;
-        for (i=2 ; i < fixnum_value(table->length) ; i += 2) {
-          sym = table->data[i];
-          if (lowtag_of(sym) == OTHER_POINTER_LOWTAG
-              && widetag_of(SYMBOL(sym)->header) == SYMBOL_WIDETAG
-              && !strcmp(name, (char*)(VECTOR(SYMBOL(sym)->name)->data)))
-              return (os_vm_address_t)fixnum_value(table->data[i+1]);
-        }
+        for (i=2 ; i < fixnum_value(table->length) ; i += 2)
+            if (lowtag_of(sym = table->data[i]) == OTHER_POINTER_LOWTAG
+                && widetag_of(SYMBOL(sym)->header) == SYMBOL_WIDETAG
+                && !strcmp(name, (char*)(VECTOR(SYMBOL(sym)->name)->data)))
+                return ALIGN_UP(offsetof(struct code,constants), 2*N_WORD_BYTES)
+                    + fixnum_value(CONS(table->data[i+1])->car) + (os_vm_address_t)code;
         // Something is wrong if we have a hashtable but find nothing.
         fprintf(stderr, "WARNING: get_asm_routine_by_name(%s) failed\n",
                 name);
