@@ -36,12 +36,8 @@
 #define BIGNUM_MARK_BIT MARK_BIT
 #endif
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
 #define interesting_pointer_p(x) \
-  (find_page_index((void*)x) >= 0 || find_immobile_page_index((void*)x) >= 0)
-#else
-#define interesting_pointer_p(x) (find_page_index((void*)x) >= 0)
-#endif
+  (find_page_index((void*)x) >= 0 || immobile_space_p(x))
 
 #ifdef DEBUG
 #  define dprintf(arg) printf arg
@@ -140,7 +136,7 @@ static lispobj gc_dequeue()
 struct hopscotch_table mark_bits;
 
 static inline uword_t compute_page_key(lispobj cons) {
-    return cons & ~(GENCGC_CARD_BYTES - 1);
+    return ALIGN_DOWN(cons, GENCGC_CARD_BYTES);
 }
 static inline int compute_dword_number(lispobj cons) {
     return (cons & (GENCGC_CARD_BYTES - 1)) >> (1+WORD_SHIFT);
@@ -380,10 +376,11 @@ void execute_full_mark_phase()
 #define timediff(b,a,field) \
     (double)((a.field.tv_sec-b.field.tv_sec)*1000000 + \
              (a.field.tv_usec-b.field.tv_usec)) / 1000000.0
-    fprintf(stderr,
-            "[Mark phase: %d pages used, HT-count=%d, ET=%f+%f sys+usr]\n",
-            (int)(page_table_pages - free_page), mark_bits.count,
-            timediff(before, after, ru_stime), timediff(before, after, ru_utime));
+    if (gencgc_verbose)
+        fprintf(stderr,
+                "[Mark phase: %d pages used, HT-count=%d, ET=%f+%f sys+usr]\n",
+                (int)(page_table_pages - free_page), mark_bits.count,
+                timediff(before, after, ru_stime), timediff(before, after, ru_utime));
 #endif
 }
 
@@ -435,16 +432,15 @@ static void sweep_fixedobj_pages(long *zeroed)
 {
     low_page_index_t page;
 
-    for (page = find_immobile_page_index((void*)(immobile_fixedobj_free_pointer-1));
-         page >= 0;
-         --page) {
+    for (page = 0 ; ; ++page) {
+        lispobj *obj = fixedobj_page_address(page);
+        if (obj >= fixedobj_free_pointer)
+            break;
         int obj_spacing = fixedobj_page_obj_align(page);
         if (!obj_spacing)
             continue;
-        char *page_base = (char*)IMMOBILE_SPACE_START + page * IMMOBILE_CARD_BYTES;
         int nwords = fixedobj_page_obj_size(page);
-        lispobj *obj = (lispobj*)page_base;
-        lispobj *limit = (lispobj*)(page_base + IMMOBILE_CARD_BYTES - obj_spacing);
+        lispobj *limit = (lispobj*)((char*)obj + IMMOBILE_CARD_BYTES - obj_spacing);
         for ( ; obj <= limit ; obj = (lispobj*)((char*)obj + obj_spacing) ) {
             lispobj header = *obj;
             if (fixnump(header)) { // is a hole
@@ -522,16 +518,18 @@ void execute_full_sweep_phase()
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     sweep_fixedobj_pages(words_zeroed);
     if (sweeplog) fprintf(sweeplog, "-- varyobj pages --\n");
-    sweep((lispobj*)IMMOBILE_VARYOBJ_SUBSPACE_START, immobile_space_free_pointer,
+    sweep((lispobj*)VARYOBJ_SPACE_START, varyobj_free_pointer,
           (uword_t)words_zeroed);
 #endif
     if (sweeplog) fprintf(sweeplog, "-- dynamic space --\n");
     walk_generation(sweep, -1, (uword_t)words_zeroed);
-    fprintf(stderr, "[Sweep phase: ");
-    int i;
-    for(i=6;i>=0;--i)
-        fprintf(stderr, "%ld%s", words_zeroed[i], i?"+":"");
-    fprintf(stderr, " words zeroed]\n");
+    if (gencgc_verbose) {
+        fprintf(stderr, "[Sweep phase: ");
+        int i;
+        for(i=6;i>=0;--i)
+            fprintf(stderr, "%ld%s", words_zeroed[i], i?"+":"");
+        fprintf(stderr, " words zeroed]\n");
+    }
     hopscotch_destroy(&mark_bits);
 #ifdef LOG_SWEEP_ACTIONS
     fclose(sweeplog);
