@@ -74,6 +74,14 @@ do {                                                                   \
 #define FUN_RAW_ADDR_OFFSET (offsetof(struct simple_fun, code) - FUN_POINTER_LOWTAG)
 #endif
 
+// For x86[-64], a simple-fun or closure's "self" slot is a fixum
+// On other backends, it is a lisp ointer.
+#if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
+#define FUN_SELF_FIXNUM_TAGGED 1
+#else
+#define FUN_SELF_FIXNUM_TAGGED 0
+#endif
+
 static inline unsigned short
 #ifdef LISP_FEATURE_64_BIT
 code_n_funs(struct code* code) { return ((code)->header >> 32) & 0x7FFF; }
