@@ -148,8 +148,7 @@
   (ctu:assert-no-consing (sb-vm::space-bytes :static)))
 
 (with-test (:name (sb-vm::map-allocated-objects :no-consing)
-                  :skipped-on :interpreter
-                  :fails-on :ppc)
+                  :skipped-on :interpreter)
   (let ((n 0))
     (sb-int:dx-flet ((f (obj type size)
                        (declare (ignore obj type size))
@@ -341,7 +340,8 @@
                       :allow-style-warnings t))))
 
 (with-test (:name :type-across-hairy-lambda-transforms)
-  (assert (subtypep (sb-kernel:%simple-fun-type (lambda (x) (find 1 (the vector x))))
+  (assert (subtypep (sb-kernel:%simple-fun-type
+                     (checked-compile `(lambda (x) (find 1 (the vector x)))))
                     '(function * (values (or (integer 1 1) null) &optional)))))
 
 (with-test (:name :lea-type-derivation)
@@ -648,3 +648,221 @@
     ;; with a toplevel value, so the second would not print enough context
     ;; because the format control and args were the same.
     (run)))
+
+(with-test (:name :cast-deletion-notes)
+  (checked-compile-and-assert
+      (:allow-notes nil)
+      `(lambda (m)
+         (setf m (list 1 2 3))
+         (the simple-vector
+              (coerce m 'vector)))
+    ((nil) #(1 2 3) :test #'equalp)))
+
+(with-test (:name :cast-deletion-notes.2)
+  (multiple-value-bind (fun fail warn style notes)
+      (checked-compile
+       `(lambda (m)
+          (setf m (list 1 2 3))
+          (the simple-vector
+               (if (vectorp m)
+                   m
+                   #(1)))))
+    (declare (ignore fail warn style))
+    (assert (equalp (funcall fun nil)
+                    #(1)))
+    (assert (= (length notes) 1))
+    (assert (typep (car notes) 'code-deletion-note))))
+
+(with-test (:name :array-call-type-deriver)
+  (checked-compile-and-assert
+      ()
+      `(lambda (vector)
+         (funcall (the (function (t t)) #'aref)
+                  vector
+                  0))
+    (((vector 333)) 333)))
+
+(with-test (:name :function-designator-cast-removal)
+  (let ((fun (checked-compile
+              `(lambda (vectors x)
+                 (declare (list vectors x))
+                 (map 'list #'svref vectors x)))))
+    (assert (notany (lambda (c)
+                      (typecase c
+                        (sb-kernel:fdefn
+                         (eq (sb-c::fdefn-name c) 'svref))
+                        (function
+                         (eq c #'svref))))
+                    (ctu:find-code-constants fun)))
+    (assert (equal (funcall fun '(#(44)) '(0)) '(44)))))
+
+(with-test (:name :zombie-casts)
+  (checked-compile-and-assert
+      ()
+      `(lambda ()
+         (flet ((f (a b)
+                  (declare (ignore a))
+                  b))
+           (multiple-value-call #'f
+             (values (the integer (unwind-protect (f 10 20)))
+                     322))))
+    (() 322)))
+
+(with-test (:name :zombie-casts.2)
+  (let ((sb-c::*max-optimize-iterations* 1))
+    (checked-compile-and-assert
+        ()
+        `(lambda (a b)
+           (declare (type fixnum a b))
+           (elt '(167992664 119771479)
+                (max 0
+                     (catch 'ct2
+                       (if (typep b '(integer -52))
+                           a
+                           0)))))
+      ((1 2) 119771479))))
+
+
+(with-test (:name :find-dfo-on-deleted-lambda)
+  (assert (= (funcall
+              (funcall (checked-compile
+                        `(lambda ()
+                           (declare (notinline <))
+                           (block nil
+                             (lambda (&key (key
+                                            (unwind-protect
+                                                 (if (< 0)
+                                                     34
+                                                     (return (catch 'c))))))
+                               key))))))
+             34)))
+
+(with-test (:name :ir1-ir2-dead-code-consistency)
+  (checked-compile-and-assert
+      ()
+      `(lambda ()
+         (loop for x below 2
+               count (zerop (min x x x x x x x x x x))))
+    (() 1)))
+
+(with-test (:name :ir1-ir2-dead-code-consistency)
+  (checked-compile-and-assert
+      ()
+      `(lambda ()
+         (loop for x below 2
+               count (zerop (min x x x x x x x x x x))))
+    (() 1)))
+
+(with-test (:name (setf svref :constant-modification))
+  (assert
+   (= (length (nth-value 2
+                         (checked-compile
+                          `(lambda (x)
+                             (setf (svref #(a b c) 1) x))
+                          :allow-warnings 'sb-int:constant-modified)))
+            1)))
+
+(with-test (:name (debug :constant-modification))
+  (assert
+   (= (length (nth-value 2
+                         (checked-compile
+                          `(lambda (x)
+                             (declare (optimize (debug 2)))
+                             (let ((m "abc"))
+                               (delete x m)))
+                          :allow-warnings 'sb-int:constant-modified)))
+      1)))
+
+(with-test (:name (debug :unused-tn-long-arglist))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n x)
+         (declare (sb-vm:word n))
+         (log (float n))
+         (nth-value 33 (funcall x . #.(loop for i to 35 collect i))))
+    ((10 (lambda (&rest args) (values-list args))) 33)))
+
+(with-test (:name (debug :unused-tn-very-long-arglist))
+  (checked-compile-and-assert
+      ()
+      `(lambda (n x)
+         (declare (sb-vm:word n))
+         (log (float n))
+         (nth-value 33 (funcall x . #.(loop for i to 350 collect i))))
+    ((10 (lambda (&rest args) (values-list args))) 33)))
+
+(with-test (:name (dynamic-extent :recursive-local-functions))
+  (checked-compile
+   `(lambda ()
+      (let ((s (labels ((%f () (%f)))
+                 (%f))))
+        (declare (dynamic-extent s))
+        (car s)))))
+
+(with-test (:name (:bogus-block &key))
+  (assert
+   (nth-value 1
+              (checked-compile `(lambda (&key (x (block 1 10))) x)
+                               :allow-failure t))))
+
+(with-test (:name :type-error-reporting)
+  (assert
+   (nth-value 1
+              (checked-compile `(lambda ()
+                                  (lambda ()
+                                    (let ((v3 0))
+                                      (cdr (1- (block b5 (if nil v3 0)))))))
+                               :allow-warnings t))))
+
+(with-test (:name :dx-on-deleted-nodes)
+  (assert
+   (nth-value 1
+              (checked-compile `(lambda ()
+                                  (restart-bind ((1 3))))
+                               :allow-warnings t))))
+
+(with-test (:name :transform-call-dfo-consistency)
+  (assert
+   (nth-value 1
+              (checked-compile
+               `(lambda ()
+                  (flet ((%f (&optional x) x))
+                    (%f)
+                    ;; Two of the %f calls are erroneous, with an extra argument
+                    (flet ((%f6 (&key (k (%f (%f -1 (%f -2 -3))))) 0))
+                      5)))
+               :allow-warnings t))))
+
+(with-test (:name :ir1-optimize-combination-dead-node)
+  (assert
+   (nth-value 1
+              (checked-compile
+               `(lambda ()
+                  (flet ((%f2 (x) x))
+                    (list (%f2 1)
+                          (multiple-value-call #'%f2 (values)))))
+               :allow-warnings t))))
+
+(with-test (:name :zombie-entry-point-reference)
+  (assert
+   (nth-value 1
+              (checked-compile
+               `(lambda () (labels ((%f ())) (%f #'%f)))
+               :allow-warnings t))))
+
+(with-test (:name (:ctypep :hairy-types))
+  (checked-compile
+   `(lambda ()
+      (the (cons (satisfies error)) '("a"))))
+  (assert
+   (nth-value 3
+              (checked-compile
+               `(lambda () (the (array abc) #()))
+               :allow-style-warnings t))))
+
+(with-test (:name (coerce :bad-type-specifier))
+  (assert
+   (nth-value 1
+              (checked-compile
+               `(lambda () (coerce (list 2) 1))
+               :allow-warnings t))))
