@@ -115,3 +115,37 @@
             (maphash
              (constantly nil)
              (make-hash-table))))))
+
+(with-test (:name :equalp-hash-float-infinity)
+  (let ((table (make-hash-table :test 'equalp)))
+    (setf (gethash sb-ext:double-float-positive-infinity table) 1
+          (gethash sb-ext:double-float-negative-infinity table) -1)
+    (dolist (v (list sb-ext:single-float-positive-infinity
+                     sb-ext:double-float-positive-infinity
+                     (complex sb-ext:single-float-positive-infinity 0)
+                     (complex sb-ext:double-float-positive-infinity 0)))
+      (assert (eql (gethash v table) 1)))
+    (dolist (v (list sb-ext:single-float-negative-infinity
+                     sb-ext:double-float-negative-infinity
+                     (complex sb-ext:single-float-negative-infinity 0)
+                     (complex sb-ext:double-float-negative-infinity 0)))
+      (assert (eql (gethash v table) -1)))))
+
+(with-test (:name (:hash equalp pathname))
+  (let* ((map (make-hash-table :test 'equalp))
+         (key  #P"some/path/"))
+    (setf (gethash key map) "my-value")
+    (format (make-broadcast-stream) "Printing: ~A~%" key)
+    (assert (remhash key map))
+    (assert (= 0 (hash-table-count map)))))
+
+(with-test (:name :clrhash-clears-rehash-p)
+  (let ((tbl (make-hash-table)))
+    (dotimes (i 10)
+      (setf (gethash (cons 'foo (gensym)) tbl) 1))
+    (gc)
+    ;; The need-to-rehash bit is set
+    (assert (eql 1 (svref (sb-impl::hash-table-table tbl) 1)))
+    (clrhash tbl)
+    ;; The need-to-rehash bit is not set
+    (assert (eql 0 (svref (sb-impl::hash-table-table tbl) 1)))))
